# -*- coding: utf-8 -*-
"""
    PyCorrFit
    This file contains TIR one component models + triplet

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""
from __future__ import division

import numpy as np

# 3D simple gauss
def CF_Gxyz_gauss(parms, tau):
    # Model 6012
    u""" Three-dimanesional free diffusion with a Gaussian laser profile
        (eliptical).

        G(τ) = offset + 1/( n*(1+τ/τ_diff) * sqrt(1 + τ/(SP²*τ_diff)) )

        Calculation of diffusion coefficient and concentration
        from the effective radius of the detection profile (r₀ = 2*σ):
        D = r₀²/(4*τ_diff)
        Conc = n/( sqrt(π³)*r₀²*z₀ )

            r₀   lateral detection radius (waist of lateral gaussian)
            z₀   axial detection length (waist of axial gaussian)
            D    Diffusion coefficient
            Conc Concentration of dye

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] n       Effective number of particles in confocal volume
        [1] τ_diff  Characteristic residence time in confocal volume
        [2] SP      SP=z₀/r₀ Structural parameter,
                    describes the axis ratio of the confocal volume
        [3] offset
        *tau* - lag time
    """
    n = parms[0]
    taudiff = parms[1]
    SP = parms[2]
    off = parms[3]

    BB = 1 / ( (1.+tau/taudiff) * np.sqrt(1.+tau/(SP**2*taudiff)) )
    G = off + 1/n * BB
    return G


# 3D blinking gauss
    # Model 6011
def CF_Gxyz_blink(parms, tau):
    u""" Three-dimanesional free diffusion with a Gaussian laser profile
        (eliptical), including a triplet component.
        The triplet factor takes into account a blinking term.
        Set *T* or *τ_trip* to 0, if no triplet component is wanted.

        G(τ) = offset + 1/( n*(1+τ/τ_diff) * sqrt(1 + τ/(SP²*τ_diff)) )
                    * ( 1+T/(1.-T)*exp(-τ/τ_trip) )

        Calculation of diffusion coefficient and concentration
        from the effective radius of the detection profile (r₀ = 2*σ):
        D = r₀²/(4*τ_diff)
        Conc = n/( sqrt(π³)*r₀²*z₀ )

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] n       Effective number of particles in confocal volume
        [1] T       Fraction of particles in triplet (non-fluorescent) state
                    0 <= T < 1
        [2] τ_trip  Characteristic residence time in triplet state
        [3] τ_diff  Characteristic residence time in confocal volume
        [4] SP      SP=z₀/r₀ Structural parameter,
                    describes the axis ratio of the confocal volume
        [5] offset
        *tau* - lag time
    """
    n = parms[0]
    T = parms[1]
    tautrip = parms[2]
    taudiff = parms[3]
    SP = parms[4]
    off = parms[5]

    if tautrip == 0 or T == 0:
        AA = 1
    else:
        AA = 1 + T/(1-T) * np.exp(-tau/tautrip)

    BB = 1 / ( (1+tau/taudiff) * np.sqrt(1+tau/(SP**2*taudiff)) )
    G = off + 1/n * AA * BB
    return G


def Check_6011(parms):
    parms[0] = np.abs(parms[0])
    T = parms[1]
    tautrip = np.abs(parms[2])
    parms[3] = np.abs(parms[3])# = taudiff
    parms[4] = np.abs(parms[4])
    #off = parms[5]
    
    # Triplet fraction is between 0 and one.
    T = (0.<=T<1.)*T + .999999999*(T>=1)

    parms[1] = T
    parms[2] = tautrip

    return parms


# 3D + 3D + Triplet Gauß
    # Model 6030
def CF_Gxyz_gauss_3D3DT(parms, tau):
    u""" Two-component three-dimensional free diffusion
        with a Gaussian laser profile, including a triplet component.
        The triplet factor takes into account a blinking term.
        Set *T* or *τ_trip* to 0, if no triplet component is wanted.

        particle1 = F₁/( (1+τ/τ₁) * sqrt(1+τ/(τ₁*SP²)))
        particle2 = α*(1-F₁)/( (1+τ/τ₂) * sqrt(1+τ/(τ₂*SP²)))
        triplet = 1 + T/(1-T)*exp(-τ/τ_trip)
        norm = (F₁ + α*(1-F₁))²
        G = 1/n*(particle1 + particle2)*triplet/norm + offset

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] n       Effective number of particles in confocal volume
                    (n = n₁+n₂)
        [1] τ₁      Diffusion time of particle species 1
        [2] τ₂      Diffusion time of particle species 2
        [3] F₁      Fraction of molecules of species 1 (n₁ = n*F₁)
                    0 <= F₁ <= 1
        [4] SP      SP=z₀/r₀, Structural parameter,
                    describes elongation of the confocal volume
        [5] α       Relative molecular brightness of particle
                    2 compared to particle 1 (α = q₂/q₁)
        [6] τ_trip  Characteristic residence time in triplet state
        [7] T       Fraction of particles in triplet (non-fluorescent) state
                    0 <= T < 1
        [8] offset
        *tau* - lag time
    """
    n=parms[0]
    taud1=parms[1]
    taud2=parms[2]
    F=parms[3]
    SP=parms[4]
    alpha=parms[5]
    tautrip=parms[6]
    T=parms[7]
    off=parms[8]

    particle1 = F/( (1+tau/taud1) * np.sqrt(1+tau/(taud1*SP**2)))
    particle2 = alpha**2*(1-F)/( (1+tau/taud2) * np.sqrt(1+tau/(taud2*SP**2)))
    # If the fraction of dark molecules is zero.
    if tautrip == 0 or T == 0:
        triplet = 1
    else:
        triplet = 1 + T/(1-T) * np.exp(-tau/tautrip)
    # For alpha == 1, *norm* becomes one
    norm = (F + alpha*(1-F))**2

    G = 1/n*(particle1 + particle2)*triplet/norm + off
    return G

def Check_3D3DT(parms):
    parms[0] = np.abs(parms[0])
    parms[1] = np.abs(parms[1]) # = taud1
    parms[2] = np.abs(parms[2]) # = taud2
    F=parms[3]
    parms[4]=np.abs(parms[4])
    parms[5]=np.abs(parms[5])
    tautrip=np.abs(parms[6])
    T=parms[7]
    #off=parms[8]
    
    
    # Triplet fraction is between 0 and one. T may not be one!
    T = (0.<=T<1.)*T + .99999999999999*(T>=1)
    # Fraction of molecules may also be one
    F = (0.<=F<=1.)*F + 1.*(F>1)

    parms[3] = F
    parms[6] = tautrip
    parms[7] = T

    return parms


def MoreInfo_1C(parms, countrate):
    # We can only give you the effective particle number
    n = parms[0]
    Info = list()
    if countrate is not None:
        # CPP
        cpp = countrate/n
        Info.append(["cpp [kHz]", cpp])
    return Info


def MoreInfo_6030(parms, countrate):
    u"""Supplementary parameters:
        [9]  n₁ = n*F₁     Particle number of species 1
        [10] n₂ = n*(1-F₁) Particle number of species 2
    """
    # We can only give you the effective particle number
    n = parms[0]
    F1 = parms[3]
    Info = list()
    # The enumeration of these parameters is very important for
    # plotting the normalized curve. Countrate must come out last!
    Info.append([u"n\u2081", n*F1])
    Info.append([u"n\u2082", n*(1.-F1)])
    if countrate is not None:
        # CPP
        cpp = countrate/n
        Info.append(["cpp [kHz]", cpp])
    return Info


# 3D Model blink gauss
m_3dblink6011 = [6011, "T+3D","3D confocal diffusion with triplet", 
                 CF_Gxyz_blink]
labels_6011 = [u"n",
               u"T",
               u"τ_trip [ms]",
               u"τ_diff [ms]",
               u"SP",
               u"offset"]
values_6011 = [4.0, 0.2, 0.001, 0.4, 5.0, 0.0]
labels_hr_6011 = [u"n",
                  u"T",
                  u"τ_trip [µs]",
                  u"τ_diff [ms]",
                  u"SP",
                  u"offset"]
factors_hr_6011 = [1., 1., 1000., 1., 1., 1.]
valuestofit_6011 = [True, True, True, True, False, False]
parms_6011 = [labels_6011, values_6011, valuestofit_6011,
              labels_hr_6011, factors_hr_6011]

# 3D Model gauss
m_3dgauss6012 = [6012, "3D","3D confocal diffusion", CF_Gxyz_gauss]
labels_6012 = [u"n",
               u"τ_diff [ms]",
               u"SP",
               u"offset"]
values_6012 = [4.0, 0.4, 5.0, 0.0]
valuestofit_6012 = [True, True, False, False]
parms_6012 = [labels_6012, values_6012, valuestofit_6012]

# 3D + 3D + T model gauss
m_gauss_3d_3d_t_mix_6030 = [6030, "T+3D+3D",
                            "Separate 3D diffusion + triplet, Gauß",
                            CF_Gxyz_gauss_3D3DT]
labels_6030  = [u"n",
                u"τ"+u"\u2081"+" [ms]",
                u"τ"+u"\u2082"+" [ms]",
                u"F"+u"\u2081", 
                u"SP",
                u"\u03b1"+" (q"+u"\u2082"+"/q"+u"\u2081"+")", 
                u"τ_trip [ms]",
                u"T",
                u"offset"
                ]
values_6030 = [ 
                25,      # n
                5,       # taud1
                1000,    # taud2
                0.5,     # F
                5,       # SP
                1.0,     # alpha
                0.001,   # tautrip
                0.01,    # T
                0.0      # offset
                ]        
# For user comfort we add values that are human readable.
# Theese will be used for output that only humans can read.
labels_human_readable_6030  = [
                        u"n",
                        u"τ"+u"\u2081"+" [ms]",
                        u"τ"+u"\u2082"+" [ms]",
                        u"F"+u"\u2081", 
                        u"SP",
                        u"\u03b1"+" (q"+u"\u2082"+"/q"+u"\u2081"+")", 
                        u"τ_trip [µs]",
                        u"T",
                        u"offset"
                            ]
values_factor_human_readable_6030 = [
                        1.,     # n
                        1.,     # taud1
                        1.,     # taud2
                        1.,     # F
                        1.,     # SP
                        1.,     # alpha
                        1000.,  # tautrip [µs]
                        1.,     # T
                        1.      # offset
                ]
valuestofit_6030 = [True, True, True, True, False, False, False, False, False]
parms_6030 = [labels_6030, values_6030, valuestofit_6030,
              labels_human_readable_6030, values_factor_human_readable_6030]


# Pack the models
model1 = dict()
model1["Parameters"] = parms_6011
model1["Definitions"] = m_3dblink6011
model1["Supplements"] = MoreInfo_1C
model1["Verification"] = Check_6011

model2 = dict()
model2["Parameters"] = parms_6012
model2["Definitions"] = m_3dgauss6012
model2["Supplements"] = MoreInfo_1C

model3 = dict()
model3["Parameters"] = parms_6030
model3["Definitions"] = m_gauss_3d_3d_t_mix_6030
model3["Supplements"] = MoreInfo_6030
model3["Verification"] = Check_3D3DT


Modelarray = [model1, model2, model3]



# -*- coding: utf-8 -*-
""" PyCorrFit

    Module tools
    This file contains useful tools, such as dialog boxes and other stuff,
    that we need in PyCorrFit.

    The tools work with triggers on page updates. Every tool has a 
    function `OnPageChanged(self, page, trigger=None)` which is called
    when something in the frontend chages. In order to minimize user
    stall time, these functions are not executed for a certain list
    of triggers that is defined in that function. This e.g. dramatically
    speeds up tools like "Statistics view" when batch fitting.
    
    Recognized triggers:
     tab_init           : initial stuff that is done for a new page
     tab_browse         : the tab has change and a new page is visible
     fit_batch          : the page is batch-fitted right now
     fit_finalize       : a (batch) fitting process is finished
     parm_batch         : parameters are changed in a batch process
     parm_finalize      : finished (batch) changing of page parameters
     page_add_batch     : when many pages are added at the same time
     page_add_finalize  : finished (batch) adding of pages

    Dimensionless representation:
    unit of time        : 1 ms
    unit of inverse time: 10³ /s
    unit of distance    : 100 nm
    unit of Diff.coeff  : 10 µm²/s
    unit of inverse area: 100 /µm²
    unit of inv. volume : 1000 /µm³

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""


# This file is necessary for this folder to become a module that can be 
# imported by PyCorrFit or other people.

import numpy as np                  # NumPy
import sys

## On Windows XP I had problems with the unicode Characters.
# I found this at 
# http://stackoverflow.com/questions/5419/python-unicode-and-the-windows-console
# and it helped:
reload(sys)
sys.setdefaultencoding('utf-8')

from . import datarange
from . import background
from . import overlaycurves
from . import batchcontrol
from . import globalfit
from . import average
from . import simulation

from . import info
from . import statistics
from . import trace
# Load all of the classes
# This also defines the order of the tools in the menu
ImpA = [ 
        ["datarange", "SelectChannels"],
        ["overlaycurves", "Wrapper_Tools"],
        ["batchcontrol", "BatchCtrl"],
        ["globalfit", "GlobalFit"],        
        ["average", "Average"],
        ["background", "BackgroundCorrection"]
       ]

ImpB = [
        ["trace", "ShowTrace"],
        ["statistics", "Stat"],
        ["info", "ShowInfo"],
        ["simulation", "Slide"]
       ]

ModuleActive = list()
ToolsActive = list()
for i in np.arange(len(ImpA)):
    # We have to add "tools." because this is a relative import
    ModuleActive.append(__import__(ImpA[i][0], globals(), locals(), [ImpA[i][1]], -1))
    ToolsActive.append(getattr(ModuleActive[i], ImpA[i][1]))

ModulePassive = list()
ToolsPassive = list()
for i in np.arange(len(ImpB)):
    ModulePassive.append(__import__(ImpB[i][0], globals(), locals(), [ImpB[i][1]], -1))
    ToolsPassive.append(getattr(ModulePassive[i], ImpB[i][1]))
    #ModulePassive.append(importlib.import_module("tools."+ImpB[i][0]))
    #ToolsPassive.append(getattr(ModulePassive[i], ImpB[i][1]))

# This is in the file menu and not needed in the dictionaries below.
from .chooseimport import ChooseImportTypes
from .chooseimport import ChooseImportTypesModel
from .comment import EditComment
# the "special" tool RangeSelector
from .parmrange import RangeSelector

ToolDict = dict()
ToolDict["A"] = ToolsActive
ToolDict["P"] = ToolsPassive


# Make the same for Menu Names in Tools
NameActive = list()
for i in np.arange(len(ImpA)):
    NameActive.append(ModuleActive[i].MENUINFO)
    
NamePassive = list()
for i in np.arange(len(ImpB)):
    NamePassive.append(ModulePassive[i].MENUINFO)


ToolName = dict()
ToolName["A"] = NameActive
ToolName["P"] = NamePassive




# -*- coding: utf-8 -*-
""" PyCorrFit

    Module tools - globalfit
    Perform global fitting on pages which share parameters.

    Dimensionless representation:
    unit of time        : 1 ms
    unit of inverse time: 10³ /s
    unit of distance    : 100 nm
    unit of Diff.coeff  : 10 µm²/s
    unit of inverse area: 100 /µm²
    unit of inv. volume : 1000 /µm³

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""


import wx
import numpy as np
from scipy import optimize as spopt

from .. import misc
from .. import models as mdls

# Menu entry name
MENUINFO = ["&Global fitting",
            "Interconnect parameters from different measurements."]

class GlobalFit(wx.Frame):
    # This tool is derived from a wx.frame.
    def __init__(self, parent):
        # Define a unique name that identifies this tool
        # Do not change this value. It is important for the Overlay tool
        # (selectcurves.py, *Wrapper_Tools*).
        self.MyName="GLOBALFIT"
        # parent is the main frame of PyCorrFit
        self.parent = parent
        # Get the window positioning correctly
        pos = self.parent.GetPosition()
        pos = (pos[0]+100, pos[1]+100)
        wx.Frame.__init__(self, parent=self.parent, title="Gobal fitting",
                 pos=pos, style=wx.DEFAULT_FRAME_STYLE|wx.FRAME_FLOAT_ON_PARENT)
        ## MYID
        # This ID is given by the parent for an instance of this class
        self.MyID = None
        # Page - the currently active page of the notebook.
        self.Page = self.parent.notebook.GetCurrentPage()
        ## Content
        self.panel = wx.Panel(self)
        self.topSizer = wx.BoxSizer(wx.VERTICAL)
        textinit = """Fitting of multiple data sets with different models.
Parameter names have to match. Select pages (e.g. 1,3-5,7),
check parameters on each page and start 'Global fit'. 
"""
        self.topSizer.Add(wx.StaticText(self.panel, label=textinit))
        ## Page selection
        self.WXTextPages = wx.TextCtrl(self.panel, value="", size=(330,-1))
        # Set initial value in text control
        pagenumlist = list()
        for i in np.arange(self.parent.notebook.GetPageCount()):
            Page = self.parent.notebook.GetPage(i)
            pagenumlist.append(int(filter(lambda x: x.isdigit(), Page.counter)))
        valstring=misc.parsePagenum2String(pagenumlist)
        self.WXTextPages.SetValue(valstring)
        self.topSizer.Add(self.WXTextPages)
        ## Weighted fitting
        # The weighted fit of the current page will be applied to
        # all other pages.
        self.weightedfitdrop = wx.ComboBox(self.panel)
        ## Bins from left and right: We also don't edit that.
        self.topSizer.Add(self.weightedfitdrop)
        ## Button
        btnfit = wx.Button(self.panel, wx.ID_ANY, 'Global fit')
        # Binds the button to the function - close the tool
        self.Bind(wx.EVT_BUTTON, self.OnFit, btnfit)
        self.topSizer.Add(btnfit)
        self.panel.SetSizer(self.topSizer)
        self.topSizer.Fit(self)
        self.SetMinSize(self.topSizer.GetMinSizeTuple())
        self.OnPageChanged(self.Page)
        # Icon
        if parent.MainIcon is not None:
            wx.Frame.SetIcon(self, parent.MainIcon)
        self.Show(True)

    
    def fit_function(self, parms):
        """
            *parms*: Parameters to fit, array
            needs: 
             self.parmstofit - list (strings) of parameters to fit
                               (corresponding to *parms*)
             self.PageData (dict with dict item = self.PageData["PageNumber"]):
                item["x"]
                item["data"]
                item["modelid"]
                item["values"]
        """
        # The list containing arrays to be minimized
        minimize = list()
        for key in self.PageData.keys():
            # Get the function
            item = self.PageData[key]
            modelid = item["modelid"]
            function = mdls.modeldict[modelid][3]
            values = self.PageData[key]["values"]
            # Set parameters for each function (Page)
            for i in np.arange(len(self.parmstofit)):
                p = self.parmstofit[i]
                labels = mdls.valuedict[modelid][0]
                if p in labels:
                    index = labels.index(p)
                    values[index] = parms[i]
            # Check parameters, if there is such a function
            check_parms = mdls.verification[modelid]
            values = check_parms(values)
            # Write parameters back?
            # self.PageData[key]["values"] = values
            # Calculate resulting correlation function
            # corr = function(item.values, item.x)
            # Subtract data. This is the function we want to minimize
            minimize.append(
              (function(values, item["x"]) - item["data"]) / item["dataweights"]
                           )

        # Flatten the list and make an array out of it.
        return np.array([it for sublist in minimize for it in sublist])


    def OnClose(self, event=None):
        # This is a necessary function for PyCorrFit.
        # Do not change it.
        self.parent.toolmenu.Check(self.MyID, False)
        self.parent.ToolsOpen.__delitem__(self.MyID)
        self.Destroy()

    def OnFit(self, e=None):
        # process a string like this: "1,2,4-9,10"
        strFull = self.WXTextPages.GetValue()
        PageNumbers = misc.parseString2Pagenum(self, strFull)
        if PageNumbers is None:
            # Something went wrong and parseString2Pagenum already displayed
            # an error message.
            return
        ## Get the corresponding pages, if they exist:
        self.PageData = dict()
        self.parmstofit = list()
        fitparms = list()
        for i in np.arange(self.parent.notebook.GetPageCount()):
            Page = self.parent.notebook.GetPage(i)
            j = filter(lambda x: x.isdigit(), Page.counter)
            if int(j) in PageNumbers:
                dataset = dict()
                try:
                    dataset["x"] = Page.dataexp[:,0]
                    dataset["data"] = Page.dataexp[:,1]
                except:
                    print "No experimental data in page #"+j+"!"
                else:
                    dataset["modelid"] = Page.modelid
                    Page.apply_parameters()
                    dataset["values"] = Page.active_parms[1]
                    # Get weights
                    weighttype = self.weightedfitdrop.GetSelection()
                    Page.Fitbox[1].SetSelection(weighttype)
                    weightname = self.weightedfitdrop.GetValue()
                    setweightname = Page.Fitbox[1].GetValue()
                    if setweightname.count(weightname) == 0:
                        print "Page "+Page.counter+" has no fitting type '"+ \
                              weightname+"'!"
                    Page.Fit_WeightedFitCheck()
                    Fitting = Page.Fit_create_instance(noplots=True)
                    if Fitting.dataweights is None:
                        dataset["dataweights"] = 1.
                    else:
                        dataset["dataweights"] = Fitting.dataweights
                    self.PageData[int(j)] = dataset
                    # Get the parameters to fit from that page
                    labels = Page.active_parms[0]
                    parms = 1*Page.active_parms[1]
                    tofit = 1*Page.active_parms[2]
                    for i in np.arange(len(labels)):
                        if tofit[i]:
                            if self.parmstofit.count(labels[i]) == 0:
                                self.parmstofit.append(labels[i])
                                fitparms.append(parms[i])
        fitparms = np.array(fitparms)
        # Now we can perform the least squares fit
        if len(fitparms) == 0:
            return
        res = spopt.leastsq(self.fit_function, fitparms[:], full_output=1)
        (popt, pcov, infodict, errmsg, ier) = res
        #self.parmoptim, self.mesg = spopt.leastsq(self.fit_function, 
        #                                          fitparms[:])
        self.parmoptim = res[0]
        # So we have the optimal parameters.
        # We would like to give each page a chi**2 and its parameters back:
        # Create a clean list of PageNumbers
        # UsedPages = dict.fromkeys(PageNumbers).keys()
        UsedPages = self.PageData.keys()
        UsedPages.sort()
        for key in UsedPages:
            # Get the Page:
            for i in np.arange(self.parent.notebook.GetPageCount()):
                aPage = self.parent.notebook.GetPage(i)
                j = filter(lambda x: x.isdigit(), aPage.counter)
                if int(j) == int(key):
                    Page = aPage
            Page.GlobalParameterShare = UsedPages
            # Get the function
            item = self.PageData[key]
            modelid = item["modelid"]
            #function = mdls.modeldict[modelid][3]
            values = 1*Page.active_parms[1]
            # Set parameters for each Page)
            for i in np.arange(len(self.parmstofit)):
                p = self.parmstofit[i]
                labels = mdls.valuedict[modelid][0]
                if p in labels:
                    index = labels.index(p)
                    values[index] = self.parmoptim[i]
                    Page.active_parms[2][index] = True
            # Check parameters, if there is such a function
            check_parms = mdls.verification[modelid]
            values = check_parms(values)
            # Write parameters back?
            Page.active_parms[1] = 1*values
            # Calculate resulting correlation function
            # corr = function(item.values, item.x)
            # Subtract data. This is the function we want to minimize
            #residual = function(values, item["x"]) - item["data"]
            # Calculate chi**2
            # Set the parameter error estimates for all pages
            minimized = self.fit_function(self.parmoptim)
            degrees_of_freedom = len(minimized) - len(self.parmoptim) - 1
            self.chi = Page.chi2 = np.sum((minimized)**2) / degrees_of_freedom
            try:
                self.covar = pcov * self.chi
            except:
                self.parmoptim_error = None
            else:
                if self.covar is not None:
                    self.parmoptim_error = np.diag(self.covar)
            p_error = self.parmoptim_error
            if p_error is None:
                Page.parmoptim_error = None
            else:
                Page.parmoptim_error = dict()
                for i in np.arange(len(p_error)):
                    Page.parmoptim_error[self.parmstofit[i]] = p_error[i]
            Page.apply_parameters_reverse()
            # Because we are plotting the weights, we need to update
            # the corresponfing info in each page:
            weightid = self.weightedfitdrop.GetSelection()
            if weightid != 0:
                # We have weights.
                # We need the following information for correct plotting.
                Page.weighted_fit_was_performed = True
                Page.weights_used_for_fitting = Fitting.dataweights
                Page.calculate_corr()
                Page.data4weight = 1.*Page.datacorr
            Page.PlotAll()


    def OnPageChanged(self, page, trigger=None):
        """
            This function is called, when something in the panel
            changes. The variable `trigger` is used to prevent this
            function from being executed to save stall time of the user.
            Forr a list of possible triggers, see the doc string of
            `tools`.
        """
        # When parent changes
        # This is a necessary function for PyCorrFit.
        # This is stuff that should be done when the active page
        # of the notebook changes.
        if trigger in ["parm_batch", "fit_batch", "page_add_batch"]:
            return
        if self.parent.notebook.GetPageCount() == 0:
            self.panel.Disable()
            return
        self.panel.Enable()
        self.Page = page
        if self.Page is not None:
            weightlist = self.Page.Fitbox[1].GetItems()
            # Do not display knot number for spline. May be different for each page.
            # Remove everything after a "(" in the weightlist string.
            # This way, e.g. the list does not show the knotnumber, which
            # we don't use anyhow.
            # We are doing this for all elements, because in the future, other (?)
            # weighting methods might be implemented.
            #for i in np.arange(len(weightlist)):
            #    weightlist[i] = weightlist[i].split("(")[0].strip()
            weightlist[1] = weightlist[1].split("(")[0].strip()
            self.weightedfitdrop.SetItems(weightlist)
            try:
                # if there is no data, this could go wrong
                self.Page.Fit_create_instance(noplots=True)
                FitTypeSelection = self.Page.Fitbox[1].GetSelection()
            except:
                FitTypeSelection = 0
            self.weightedfitdrop.SetSelection(FitTypeSelection)
            ## Knotnumber: we don't want to interfere
            # The user might want to edit the knotnumbers.
            # self.FitKnots = Page.FitKnots   # 5 by default

    def SetPageNumbers(self, pagestring):
        self.WXTextPages.SetValue(pagestring)



# -*- coding: utf-8 -*-
""" Wrapper for Loading PicoQuant .pt3 data files

Wraps around FCS_point_correlator by Dominic Waithe
https://github.com/dwaithe/FCS_point_correlator
"""
import numpy as np
import os
from .read_pt3_scripts.correlation_objects import picoObject


class ParameterClass():
	"""Stores parameters for correlation """
	def __init__(self):
		
		#Where the data is stored.
		self.data = []
		self.objectRef =[]
		self.subObjectRef =[]
		self.colors = ['blue','green','red','cyan','magenta','yellow','black']
		self.numOfLoaded = 0
		self.NcascStart = 0
		self.NcascEnd = 25
		self.Nsub = 6
		self.winInt = 10
		self.photonCountBin = 25
        

def openPT3(dirname, filename):
    """ Retreive correlation curves from PicoQuant data files 
    
    This function is a wrapper around the PicoQuant capability of
    FCS_Viewer by Dominic Waithe.
    """
    par_obj = ParameterClass()
    
    pt3file = picoObject(os.path.join(dirname, filename), par_obj, None)

    po = pt3file

    auto = po.autoNorm
    # lag time [ms]
    autotime = po.autotime.reshape(-1)

    corrlist = list()
    tracelist = list()
    typelist = list()
    
    # Some data points are zero for some reason
    id1 = np.where(autotime!=0)


    # AC0 - autocorrelation CH0
    corrac0 = auto[:,0,0]    
    if np.sum(np.abs(corrac0[id1])) != 0:
        typelist.append("AC0")
        # autotime,auto[:,0,0]
        corrlist.append(np.hstack( (autotime[id1].reshape(-1,1),
                                    corrac0[id1].reshape(-1,1)) ))
    
    # AC1 - autocorrelation CH1
    corrac1 = auto[:,1,1]
    if np.sum(np.abs(corrac1[id1])) != 0:
        typelist.append("AC1")
        # autotime,auto[:,1,1]
        corrlist.append(np.hstack( (autotime[id1].reshape(-1,1),
                                    corrac1[id1].reshape(-1,1)) ))
    
    # CC01 - Cross-Correlation CH0-CH1
    corrcc01 = auto[:,0,1]
    if np.sum(np.abs(corrcc01[id1])) != 0:
        typelist.append("CC01")
        # autotime,auto[:,0,1]
        corrlist.append(np.hstack( (autotime[id1].reshape(-1,1),
                                    corrcc01[id1].reshape(-1,1)) ))
    
    # CC10 - Cross-Correlation CH1-CH0
    corrcc10 = auto[:,1,0]
    if np.sum(np.abs(corrcc10[id1])) != 0:
        typelist.append("CC10")
        # autotime,auto[:,1,0]
        corrlist.append(np.hstack( (autotime[id1].reshape(-1,1),
                                    corrcc10[id1].reshape(-1,1)) ))

    dictionary = dict()
    dictionary["Correlation"] = corrlist
    dictionary["Trace"] = tracelist
    dictionary["Type"] = typelist
    filelist = list()
    for i in typelist:
        filelist.append(filename)
        tracelist.append(None)
    dictionary["Filename"] = filelist

    return dictionary



# -*- coding: utf-8 -*-
""" PyCorrFit

    Module tools - comment
    Just edit the sessions comment.

    Dimensionless representation:
    unit of time        : 1 ms
    unit of inverse time: 10³ /s
    unit of distance    : 100 nm
    unit of Diff.coeff  : 10 µm²/s
    unit of inverse area: 100 /µm²
    unit of inv. volume : 1000 /µm³

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""


import wx


class EditComment(wx.Frame):
    """ Little Dialog to edit the comment on the session. """
    def __init__(self, parent):
        ## Variables
        # parent is main frame
        self.parent = parent
        # Get the window positioning correctly
        pos = self.parent.GetPosition()
        pos = (pos[0]+100, pos[1]+100)
        wx.Frame.__init__(self, parent=parent, title="Session comment",
                 pos=pos, style=wx.DEFAULT_FRAME_STYLE|wx.FRAME_FLOAT_ON_PARENT)
        initial_size = (400,300)
        initial_sizec = (initial_size[0], initial_size[1]-50)
        self.SetSize(initial_size)
        self.SetMinSize((400,300))
        ## Content
        self.panel = wx.Panel(self)
        self.control = wx.TextCtrl(self.panel, style=wx.TE_MULTILINE, 
                        size=initial_sizec, value=self.parent.SessionComment)
        text = wx.StaticText(self.panel, 
                   label="Session comments will be saved in the  session file.")
        # buttons
        btnclose = wx.Button(self.panel, wx.ID_ANY, 'Close')
        btnokay = wx.Button(self.panel, wx.ID_ANY, 'OK')
        self.Bind(wx.EVT_BUTTON, self.OnClose, btnclose)
        self.Bind(wx.EVT_BUTTON, self.OnOkay, btnokay)
        #sizers
        self.topSizer = wx.BoxSizer(wx.VERTICAL)
        buttonsizer = wx.BoxSizer(wx.HORIZONTAL)
        buttonsizer.Add(btnclose, 1)
        buttonsizer.Add(btnokay, 1)
        self.topSizer.Add(text)
        self.topSizer.Add(buttonsizer)
        self.topSizer.Add(self.control)
        self.panel.SetSizer(self.topSizer)
        self.topSizer.Fit(self)
        #Icon
        if parent.MainIcon is not None:
            wx.Frame.SetIcon(self, parent.MainIcon)
        self.Show(True)
        wx.EVT_SIZE(self, self.OnSize)


    def OnSize(self, event):
        size = event.GetSize()
        sizec = (size[0], size[1]-50)
        self.panel.SetSize(size)
        self.control.SetSize(sizec)


    def OnClose(self, event=None):
        self.parent.filemenu.Check(self.parent.menuComm.GetId(), False)
        self.Destroy()


    def OnOkay(self, event):
        self.parent.SessionComment = self.control.GetValue()
        self.OnClose()




# -*- coding: utf-8 -*-
""" PyCorrFit

    Module tools - info
    Open a text window with lots of information.

    Dimensionless representation:
    unit of time        : 1 ms
    unit of inverse time: 10³ /s
    unit of distance    : 100 nm
    unit of Diff.coeff  : 10 µm²/s
    unit of inverse area: 100 /µm²
    unit of inv. volume : 1000 /µm³

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""


import wx
import numpy as np

from .. import fitting
from .. import models as mdls

# Menu entry name
MENUINFO = ["Page &info",
            "Display some information on the current page."]
            
class InfoClass(object):
    """ This class get's all the Info possible from a Page and
        makes it available through a dictionary with headings as keys.
    """
    def __init__(self, CurPage=None, Pagelist=None ):
        # A list of all Pages currently available:
        self.Pagelist = Pagelist
        # The current page we are looking at:
        self.CurPage = CurPage


    def GetAllInfo(self):
        """ Get a dictionary with page titles and an InfoDict as value.
        """
        MultiInfo = dict()
        for Page in self.Pagelist:
            # Page counter includes a whitespace and a ":" which we do not want.
            MultiInfo[Page.counter[:-2]] = self.GetPageInfo(Page)
        return MultiInfo


    def GetCurInfo(self):
        """ Get all the information about the current Page.
            Added for convenience. You may use GetPageInfo.
        """
        return self.GetPageInfo(self.CurPage)


    def GetCurFancyInfo(self):
        """ For convenience. """
        return self.GetFancyInfo(self.CurPage)


    def GetFancyInfo(self, Page):
        """ Get a nice string representation of the Info """
        InfoDict = self.GetPageInfo(Page)
        # Version
        Version = "PyCorrFit v."+InfoDict["version"][0]+"\n"
        # Title
        Title = "\n"
        for item in InfoDict["title"]:
            Title = Title + item[0]+"\t"+ item[1]+"\n"
        # Parameters
        Parameters = "\nParameters:\n"
        for item in InfoDict["parameters"]:
            Parameters = Parameters + "  "+item[0]+"\t"+ str(item[1])+"\n"
        # Supplementary parameters
        Supplement = "\nSupplementary parameters:\n"
        try:
            for item in InfoDict["supplement"]:
                Supplement = Supplement + "  "+item[0]+"\t"+ str(item[1])+"\n"
        except KeyError:
            Supplement = ""
        # Fitting
        Fitting = "\nFitting:\n"
        try:
            for item in InfoDict["fitting"]:
                Fitting = Fitting + "  "+item[0]+"\t"+str(item[1])+"\n"
        except KeyError:
            Fitting = ""
        # Background
        Background = "\nBackground:\n"
        try:
            for item in InfoDict["background"]:
                Background = Background + "  "+item[0]+"\t"+str(item[1])+"\n"
        except KeyError:
            Background = ""

        # Function doc string
        ModelDoc = "\n\nModel doc string:\n       " + InfoDict["modeldoc"][0]
        # Supplementary variables
        try:
            SupDoc = "\n"+8*" "+InfoDict["modelsupdoc"][0]
        except:
            SupDoc = ""
        PageInfo = Version+Title+Parameters+Supplement+Fitting+Background+\
                   ModelDoc+SupDoc
        return PageInfo


    def GetPageInfo(self, Page):
        """ Needs a Page and gets all information from it """
        Page.PlotAll("init")
        # A dictionary with headings as keys and lists of singletts/tuples as 
        # values. If it is a tuple, it might me interesting for a table.
        InfoDict = dict()
        # Get model information
        model = [Page.model, Page.tabtitle.GetValue(), Page.modelid]
        parms = Page.active_parms[1]
        fct = Page.active_fct.__name__
        InfoDict["version"] = [Page.parent.version]
        Title = list()
        # The tool statistics relys on the string "filename/title".
        # Do not change it!
        if len(model[1]) == 0:
            # Prevent saving no title
            model[1] = "NoName"
        Title.append(["filename/title", model[1] ]) 
        Title.append(["Model ID", str(model[2]) ]) 
        Title.append(["Model name", model[0] ]) 
        Title.append(["Model function", fct ]) 
        Title.append(["Page number", Page.counter[1:-2] ]) 
        ## Parameters
        Parameters = list()
        # Use this function to determine human readable parameters, if possible
        Units, Newparameters = mdls.GetHumanReadableParms(model[2], parms)
        # Add Parameters
        for i in np.arange(len(parms)):
            Parameters.append([ Units[i], Newparameters[i] ])
        InfoDict["parameters"] = Parameters
        # Add some more information if available
        # Info is a dictionary or None
        MoreInfo = mdls.GetMoreInfo(model[2], Page)
        if MoreInfo is not None:
            InfoDict["supplement"] = MoreInfo
            # Try to get the dictionary entry of a model
            try:
                # This function should return all important information
                # that can be calculated from the given parameters.
                func_info = mdls.supplement[model[2]]
            except KeyError:
                # No information available
                pass
            else:
                InfoDict["modelsupdoc"] = [func_info.func_doc]
        ## Fitting
        alg = fitting.Algorithms[Page.fit_algorithm][1]
        weightedfit = Page.weighted_fit_was_performed
        weightedfit_type = Page.weighted_fittype
        fittingbins = Page.weighted_nuvar  # from left and right
        Fitting = list()
        if Page.dataexp is not None:
            # Mode AC vs CC
            if Page.IsCrossCorrelation is True:
                Title.append(["Type AC/CC", "Cross-correlation" ]) 
            else:
                Title.append(["Type AC/CC", "Autocorrelation" ]) 
            Fitting.append([ u"χ²", Page.chi2 ])
            if Page.weighted_fit_was_performed:
                Chi2type = u"Weighted sum of squares"
            else:
                Chi2type = u"Sum of squares"
            Fitting.append([ u"χ²-type", Chi2type ])
            Fitting.append([ "Weighted fit", weightedfit_type ])
            Fitting.append([ "Algorithm", alg ])
            if len(Page.GlobalParameterShare) != 0:
                shared = str(Page.GlobalParameterShare[0])
                for item in Page.GlobalParameterShare[1:]:
                    shared += ", "+str(item)
                Fitting.append(["Shared parameters with Pages", shared])
            if weightedfit is True:
                Fitting.append([ "Std. channels", 2*fittingbins+1 ])
            # Fitting range:
            t1 = 1.*Page.taufull[Page.startcrop]
            t2 = 1.*Page.taufull[Page.endcrop-1]
            Fitting.append([ "Interval start [ms]", "%.4e" % t1 ])
            Fitting.append([ "Interval end [ms]", "%.4e" % t2 ])
            # Fittet parameters and errors
            somuch = sum(Page.active_parms[2])
            if somuch >= 1:
                fitted = ""
                for i in np.arange(len(Page.active_parms[2])):
                    if np.bool(Page.active_parms[2][i]) is True:
                        errorvar = Page.active_parms[0][i] # variable name
                        fitted=fitted+errorvar+ ", "
                fitted = fitted.strip().strip(",") # remove trailing comma
                Fitting.append(["fit par.", fitted])
                # Fitting error included in v.0.7.3
                Errors_fit = Page.parmoptim_error
                if Errors_fit is not None:
                    errkeys = Errors_fit.keys()
                    errkeys.sort()
                    for key in errkeys:
                        savekey, saveval = \
                            mdls.GetHumanReadableParameterDict(model[2],
                                                [key], [Errors_fit[key]])
                        # The tool statistics relys on the string "Err ".
                        # Do not change it!
                        Fitting.append(["Err "+savekey[0], saveval[0]])
            InfoDict["fitting"] = Fitting
        ## Normalization
        if Page.normparm is None:
            normparmtext = "None"
        elif Page.normparm < len(Page.active_parms[0]):
            normparmtext = Page.active_parms[0][Page.normparm]
        else:
            # supplementary parameters
            supnum = Page.normparm - len(Page.active_parms[1])
            normparmtext =  MoreInfo[supnum][0]
        Title.append(["Normalization", normparmtext ]) 
        ## Background
        Background = list()
        if Page.IsCrossCorrelation:
            if ( Page.bgselected is not None and
                 Page.bg2selected is not None     ):
                # Channel 1
                bgname = Page.parent.Background[Page.bgselected][1]
                if len(bgname) == 0:
                    # Prevent saving no name
                    bgname = "NoName"
                Background.append([ "bg name Ch1", bgname])
                Background.append([ "bg rate Ch1 [kHz]", 
                           Page.parent.Background[Page.bgselected][0] ])
                # Channel 2
                bg2name = Page.parent.Background[Page.bg2selected][1]
                if len(bg2name) == 0:
                    # Prevent saving no name
                    bg2name = "NoName"
                Background.append([ "bg name Ch2", bg2name])
                Background.append([ "bg rate Ch2 [kHz]", 
                          Page.parent.Background[Page.bg2selected][0] ])
                InfoDict["background"] = Background
        else:
            if Page.bgselected is not None:
                bgname = Page.parent.Background[Page.bgselected][1]
                if len(bgname) == 0:
                    # Prevent saving no name
                    bgname = "NoName"
                bgrate = Page.parent.Background[Page.bgselected][0]
                Background.append([ "bg name", bgname ])
                Background.append([ "bg rate [kHz]", bgrate ])
                InfoDict["background"] = Background
        ## Function doc string
        InfoDict["modeldoc"] = [Page.active_fct.func_doc]
        InfoDict["title"] = Title

        return InfoDict


class ShowInfo(wx.Frame):
    def __init__(self, parent):
        # parent is main frame
        self.parent = parent
        # Get the window positioning correctly
        pos = self.parent.GetPosition()
        pos = (pos[0]+100, pos[1]+100)
        wx.Frame.__init__(self, parent=self.parent, title="Info",
                 pos=pos, style=wx.DEFAULT_FRAME_STYLE|wx.FRAME_FLOAT_ON_PARENT)
        ## MYID
        # This ID is given by the parent for an instance of this class
        self.MyID = None
        # Page
        self.Page = self.parent.notebook.GetCurrentPage()
        # Size
        initial_size = wx.Size(650,700)
        initial_sizec = (initial_size[0]-6, initial_size[1]-30)
        self.SetMinSize(wx.Size(200,200))
        self.SetSize(initial_size)
        ## Content
        self.panel = wx.Panel(self)
        self.control = wx.TextCtrl(self.panel, style=wx.TE_MULTILINE, 
                        size=initial_sizec)
        self.control.SetEditable(False)
        font1 = wx.Font(10, wx.MODERN, wx.NORMAL, wx.NORMAL, False, u'Monospace')
        self.control.SetFont(font1)
        btncopy = wx.Button(self.panel, wx.ID_CLOSE, 'Copy to clipboard')
        self.Bind(wx.EVT_BUTTON, self.OnCopy, btncopy)
        self.topSizer = wx.BoxSizer(wx.VERTICAL)
        self.topSizer.Add(btncopy)
        self.topSizer.Add(self.control)
        self.panel.SetSizer(self.topSizer)
        self.topSizer.Fit(self)
        #Icon
        if parent.MainIcon is not None:
            wx.Frame.SetIcon(self, parent.MainIcon)
        self.Show(True)
        wx.EVT_SIZE(self, self.OnSize)
        self.Content()


    def Content(self):
        # Fill self.control with content.
        # Parameters and models
        if self.parent.notebook.GetPageCount() == 0:
            self.control.SetValue("")
            self.panel.Disable()
            return
        self.panel.Enable()
        Page = self.Page
        InfoMan = InfoClass(CurPage=Page)
        PageInfo = InfoMan.GetCurFancyInfo()
        self.control.SetValue(PageInfo)


    def OnClose(self, event=None):
        self.parent.toolmenu.Check(self.MyID, False)
        self.parent.ToolsOpen.__delitem__(self.MyID)
        self.Destroy()


    def OnCopy(self, event):
        if not wx.TheClipboard.IsOpened():
            clipdata = wx.TextDataObject()
            clipdata.SetText(self.control.GetValue())
            wx.TheClipboard.Open()
            wx.TheClipboard.SetData(clipdata)
            wx.TheClipboard.Close()
        else:
            print "Other application has lock on clipboard."


    def OnPageChanged(self, page=None, trigger=None):
        """
            This function is called, when something in the panel
            changes. The variable `trigger` is used to prevent this
            function from being executed to save stall time of the user.
            Forr a list of possible triggers, see the doc string of
            `tools`.
        """
        if trigger in ["parm_batch", "fit_batch", "page_add_batch"]:
            return
        # When parent changes
        self.Page = page
        self.Content()


    def OnSize(self, event):
        size = event.GetSize()
        sizec = wx.Size(size[0]-5, size[1]-30)
        self.panel.SetSize(size)
        self.control.SetSize(sizec)



# -*- coding: utf-8 -*-
""" 
    PyCorrFit
    
    functions in this file: *openASC*

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""
from __future__ import division

import os
import csv
import numpy as np


def openASC(dirname, filename):
    """ Read data from a .ASC file, created by
        some ALV-6000 correlator.

            ALV-6000/E-WIN Data
            Date :	"2/20/2012"
            ...
            "Correlation"
              1.25000E-004	  3.00195E-001
              2.50000E-004	  1.13065E-001
              3.75000E-004	  7.60367E-002
              5.00000E-004	  6.29926E-002
              6.25000E-004	  5.34678E-002
              7.50000E-004	  4.11506E-002
              8.75000E-004	  4.36752E-002
              1.00000E-003	  4.63146E-002
              1.12500E-003	  3.78226E-002
            ...
              3.35544E+004	 -2.05799E-006
              3.77487E+004	  4.09032E-006
              4.19430E+004	  4.26295E-006
              4.61373E+004	  1.40265E-005
              5.03316E+004	  1.61766E-005
              5.45259E+004	  2.19541E-005
              5.87202E+004	  3.26527E-005
              6.29145E+004	  2.72920E-005

            "Count Rate"
               1.17188	      26.77194
               2.34375	      26.85045
               3.51563	      27.06382
               4.68750	      26.97932
               5.85938	      26.73694
               7.03125	      27.11332
               8.20313	      26.81376
               9.37500	      26.82741
              10.54688	      26.88801
              11.71875	      27.09710
              12.89063	      27.13209
              14.06250	      27.02200
              15.23438	      26.95287
              16.40625	      26.75657
              17.57813	      26.43056
            ...
             294.14063	      27.22597
             295.31250	      26.40581
             296.48438	      26.33497
             297.65625	      25.96457
             298.82813	      26.71902

        1. We are interested in the "Correlation" section,
        where the first column denotes tau in ms and the second row the
        correlation signal. Values are separated by a tabulator "\t" (some " ").

        2. We are also interested in the "Count Rate" section. Here the times
        are saved as seconds and not ms like above.

        3. There is some kind of mode where the ALV exports five runs at a
        time and averages them. The sole correlation data is stored in the
        file, but the trace is only stored as average or something.
        So I would not recommend this. However, I added support for this.
        PyCorrFit then only imports the average data.
         ~ Paul, 2012-02-20
        Correlation data starts at "Correlation (Multi, Averaged)".

        Returns:
        [0]:
         An array with tuples containing two elements:
         1st: tau in ms
         2nd: corresponding correlation signal
        [1]:
         Intensity trace:
         1st: time in ms
         2nd: Trace in kHz
        [2]:
         An array with N elements, indicating, how many curves we are opening
         from the file. Elements can be names and must be convertible to
         strings.
    """
    openfile = open(os.path.join(dirname, filename), 'r')
    Alldata = openfile.readlines()
    # End of trace
    EndT = Alldata.__len__()
    ## Correlation function
    # Find out where the correlation function is
    for i in np.arange(len(Alldata)):
        if Alldata[i].startswith('Mode'):
            mode = Alldata[i][5:].strip(' ":').strip().strip('"')
            if mode.lower().count('single'):
                single = True
                channel = mode.split(" ")[-1]
            else:
                # dual
                single = False
            # accc ?
            if mode.lower().count("cross") == 1:
                accc = "CC"
            else:
                accc = "AC"
        if Alldata[i].startswith('"Correlation'):
            # This tells us if there is only one curve or if there are
            # multiple curves with an average.
            if (Alldata[i].strip().lower() == 
                '"correlation (multi, averaged)"' ):
                multidata = True
            else:
                multidata = False
        if Alldata[i].startswith('"Correlation"'):
            # Start of correlation function
            StartC = i+1
        if Alldata[i].startswith('"Correlation (Multi, Averaged)"'):
            # Start of AVERAGED correlation function !!!
            # There are several curves now.
            StartC = i+2
        if Alldata[i].replace(" ", "").lower().strip() == '"countrate"':
            # takes cate of "Count Rate" and "Countrate"
            # End of correlation function
            EndC = i-1
            # Start of trace (goes until end of file)
            StartT = i+1
        if Alldata[i].startswith('Monitor Diode'):
            EndT = i-1
    # Get the header
    Namedata = Alldata.__getslice__(StartC-1, StartC)
    ## Define *curvelist*
    curvelist = csv.reader(Namedata, delimiter='\t').next()
    if len(curvelist) <= 2:
        # Then we have just one single correlation curve
        curvelist = [""]
    else:
        # We have a number of correlation curves. We need to specify
        # names for them. We take these names from the headings.
        # Lag times not in the list
        curvelist.remove(curvelist[0])
        # Last column is empty
        curvelist.remove(curvelist[-1])
    ## Correlation function
    Truedata = Alldata.__getslice__(StartC, EndC)
    readdata = csv.reader(Truedata, delimiter='\t')
    data = list()
    # Add lists to *data* according to the length of *curvelist*
    for item in curvelist:
        data.append(list())
    # Work through the rows in the read data
    for row in readdata:
        for i in np.arange(len(curvelist)):
            if len(row) > 0:
                data[i].append( (np.float(row[0]), np.float(row[i+1])) )
    ## Trace
    # Trace is stored in two columns
    # 1st column: time [s]
    # 2nd column: trace [kHz] 
    # Get the trace
    Tracedata = Alldata.__getslice__(StartT, EndT)
    timefactor = 1000 # because we want ms instead of s
    readtrace = csv.reader(Tracedata, delimiter='\t')
    trace = list()
    trace2 = list()
    # Work through the rows
    for row in readtrace:
        # time in ms, countrate
        trace.append(list())
        trace[0].append((np.float(row[0])*timefactor,
                         np.float(row[1])))
        # Only trace[0] contains the trace!
        for i in np.arange(len(curvelist)-1):
            trace.append(list())
            trace[i+1].append((np.float(row[0])*timefactor, 0))
        if not single:
            k = len(curvelist)/2
            if int(k) != k:
                print "Problem with ALV data. Single mode not recognized."
            # presumably dual mode. There is a second trace
            # time in ms, countrate
            trace2.append(list())
            trace2[0].append((np.float(row[0])*timefactor,
                              np.float(row[2])))
            # Only trace2[0] contains the trace!
            for i in np.arange(len(curvelist)-1):
                trace2.append(list())
                trace2[i+1].append((np.float(row[0])*timefactor, 0))
    # return as an array
    openfile.close()

    # group the resulting curves
    corrlist = list()
    tracelist = list()
    typelist = list()
    
        
    if single:
        # We only have several runs and one average
        # split the trace into len(curvelist)-1 equal parts
        if multidata:
            nav = 1
        else:
            nav = 0
        splittrace = mysplit(trace[0], len(curvelist)-nav)
        i = 0
        for t in range(len(curvelist)):
            typ = curvelist[t]
            if typ.lower()[:7] == "average":
                typelist.append("{} average".format(channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(np.array(trace[0]))
            else:
                typelist.append("{} {}".format(accc, channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(splittrace[i])
                i += 1
    elif accc == "AC":
        # Dual mode, autocorrelation
        # We now have two averages and two different channels.
        # We now have two traces.
        # The data is assembled in blocks. That means the first block
        # contains an average and the data of channel 0 and the second
        # block contains data and average of channel 1. We can thus
        # handle the data from 0 to len(curvelist)/2 and from
        # len(curvelist)/2 to len(curvelist) as two separate data sets.
        # CHANNEL 0
        if multidata:
            nav = 1
        else:
            nav = 0
        channel = "CH0"
        splittrace = mysplit(trace[0], len(curvelist)/2-nav)
        i = 0
        for t in range(int(len(curvelist)/2)):
            typ = curvelist[t]
            if typ.lower()[:7] == "average":
                typelist.append("{} average".format(channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(np.array(trace[0]))
            else:
                typelist.append("{} {}".format(accc, channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(splittrace[i])
                i += 1
        # CHANNEL 1
        channel = "CH1"
        splittrace2 = mysplit(trace2[0], len(curvelist)/2-nav)
        i = 0
        for t in range(int(len(curvelist)/2),int(len(curvelist))):
            typ = curvelist[t]
            if typ.lower()[:7] == "average":
                typelist.append("{} average".format(channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(np.array(trace2[0]))
            else:
                typelist.append("{} {}".format(accc, channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(splittrace2[i])
                i += 1
    elif accc == "CC":
        if multidata:
            nav = 1
        else:
            nav = 0
        # Dual mode, cross-correlation
        channel = "CC01"
        splittrace = mysplit(trace[0], len(curvelist)/2-nav)
        splittrace2 = mysplit(trace2[0], len(curvelist)/2-nav)
        i = 0
        for t in range(int(len(curvelist)/2)):
            typ = curvelist[t]
            if typ.lower()[:7] == "average":
                typelist.append("{} average".format(channel))
                corrlist.append(np.array(data[t]))
                tracelist.append([np.array(trace[0]),
                                  np.array(trace2[0])  ])
            else:
                typelist.append("{} {}".format(accc, channel))
                corrlist.append(np.array(data[t]))
                tracelist.append([splittrace[i], splittrace2[i]])
                i += 1
        # CHANNEL 1
        channel = "CC10"
        i = 0
        for t in range(int(len(curvelist)/2),int(len(curvelist))):
            typ = curvelist[t]
            if typ.lower()[:7] == "average":
                typelist.append("{} average".format(channel))
                corrlist.append(np.array(data[t]))
                # order must be the same as above
                tracelist.append([np.array(trace[0]),
                                  np.array(trace2[0])  ])
            else:
                typelist.append("{} {}".format(accc, channel))
                corrlist.append(np.array(data[t]))
                # order must be the same as above
                tracelist.append([splittrace[i], splittrace2[i]])
                i += 1
    else:
        print "Could not detect data file format for: {}".format(filename)
        corrlist = np.array(data)
        tracelist = np.array(trace)
        typelist = curvelist

    dictionary = dict()
    dictionary["Correlation"] = corrlist
    dictionary["Trace"] = tracelist
    dictionary["Type"] = typelist
    filelist = list()
    for i in curvelist:
        filelist.append(filename)
    dictionary["Filename"] = filelist
    return dictionary


def mysplit(a, n):
    """
       Split a trace into n equal parts by interpolation.
       The signal average is preserved, but the signal variance will
       decrease.
    """
    if n == 1:
        return [np.array(a)]
    a = np.array(a)
    N = len(a)
    lensplit = np.int(np.ceil(N/n))

    # xp is actually rounded -> recalculate
    xp, step = np.linspace(a[:,0][0], a[:,0][-1], N,
                           endpoint=True, retstep=True)
    
    # let xp start at zero
    xp -= a[:,0][0]
    yp = a[:,1]
    
    # time frame for each new curve
    #dx = xp[-1]/n

    # perform interpolation of new trace
    x, newstep = np.linspace(0, xp[-1], lensplit*n,
                        endpoint=True, retstep=True)
    # interpolating reduces the variance and possibly changes the avg
    y = np.interp(x,xp,yp)
    
    data = np.zeros((lensplit*n,2))
    data[:,0] = x + newstep
    # make sure that the average stays the same:
    data[:,1] = y - np.average(y) + np.average(yp)
    
    return np.split(data,n)
    



# -*- coding: utf-8 -*-
"""
    PyCorrFit
    This file contains TIR one component models + Triplet

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""
import numpy as np                  # NumPy
import scipy.special as sps

def wixi(x):
    """ Complex Error Function (Faddeeva/Voigt).
        w(i*x) = exp(x**2) * ( 1-erf(x) )
        This function is called by other functions within this module.
        We are using the scipy.special.wofz module which calculates
        w(z) = exp(-z**2) * ( 1-erf(-iz) )
        z = i*x
    """
    z = x*1j
    wixi = sps.wofz(z)
    # We should have a real solution. Make sure nobody complains about
    # some zero-value imaginary numbers.
    
    return np.real_if_close(wixi)

def CF_Gxyz_TIR_gauss(parms, tau):
    u""" Three-dimensional free diffusion with a Gaussian lateral 
        detection profile and an exponentially decaying profile
        in axial direction.

        x = sqrt(D*τ)*κ
        κ = 1/d_eva
        w(i*x) = exp(x²)*erfc(x)
        gz = κ * [ sqrt(D*τ/π) + (1 - 2*D*τ*κ)/(2*κ) * w(i*x) ]
        g2D = 1 / [ π (r₀² + 4*D*τ) ]
        G = 1/C_3D * g2D * gz


        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] D      Diffusion coefficient
        [1] r₀     Lateral extent of the detection volume
        [2] d_eva  Evanescent field depth
        [3] C_3D   Particle concentration in the confocal volume
        *tau* - lag time
    """
    # model 6013
    D = parms[0]
    r0 = parms[1]
    deva = parms[2]
    Conc = parms[3]

    # Calculate sigma: width of the gaussian approximation of the PSF
    Veff = np.pi * r0**2 * deva
    Neff = Conc * Veff

    taudiff = r0**2/(4*D)
    # 2D gauss component
    # G2D = 1/N2D * g2D = 1/(Aeff*Conc.2D) * g2D
    g2D = 1 / ( (1.+tau/taudiff) )

    # 1d TIR component
    # Axial correlation    
    kappa = 1/deva
    x = np.sqrt(D*tau)*kappa
    w_ix = wixi(x)

    # Gz = 1/N1D * gz = kappa / Conc.1D * gz
    gz = kappa * (np.sqrt(D*tau/np.pi) - (2*D*tau*kappa**2 - 1)/(2*kappa) * w_ix)

    # gz * g2D * 1/( deva *A2D) * 1 / Conc3D

    # Neff is not the actual particle number. This formula just looks nicer
    # this way.
    # What would be easier to get is:
    # 1 / (Conc * deva * np.pi * r0) * gz * g2D

    return 1 / (Neff) * g2D * gz
    


def CF_Gxyz_TIR_gauss_trip(parms, tau):
    u""" Three-dimensional free diffusion with a Gaussian lateral 
        detection profile and an exponentially decaying profile
        in axial direction, including a triplet component.

        x = sqrt(D*τ)*κ
        κ = 1/d_eva
        w(i*x) = exp(x²)*erfc(x)
        gz = κ * [ sqrt(D*τ/π) + (1 - 2*D*τ*κ)/(2*κ) * w(i*x) ]
        g2D = 1 / [ π (r₀² + 4*D*τ) ]
        triplet = 1 + T/(1-T)*exp(-τ/τ_trip)
        G = 1/C_3D * g2D * gz * triplet


        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] D      Diffusion coefficient
        [1] r₀     Lateral extent of the detection volume
        [2] d_eva  Evanescent field depth
        [3] C_3D   Particle concentration in the confocal volume
        [4] τ_trip  Characteristic residence time in triplet state
        [5] T       Fraction of particles in triplet (non-fluorescent) state
                    0 <= T < 1
        *tau* - lag time
    """
    # model 6014
    D = parms[0]
    r0 = parms[1]
    deva = parms[2]
    Conc = parms[3]
    tautrip=parms[4]
    T=parms[5]

    # Calculate sigma: width of the gaussian approximation of the PSF
    Veff = np.pi * r0**2 * deva
    Neff = Conc * Veff

    taudiff = r0**2/(4*D)
    # 2D gauss component
    # G2D = 1/N2D * g2D = 1/(Aeff*Conc.2D) * g2D
    g2D = 1 / ( (1.+tau/taudiff) )

    # 1d TIR component
    # Axial correlation    
    kappa = 1/deva
    x = np.sqrt(D*tau)*kappa
    w_ix = wixi(x)

    # Gz = 1/N1D * gz = kappa / Conc.1D * gz
    gz = kappa * (np.sqrt(D*tau/np.pi) - (2*D*tau*kappa**2 - 1)/(2*kappa) * w_ix)

    ### triplet
    if tautrip == 0 or T == 0:
        triplet = 1
    else:
        triplet = 1 + T/(1-T) * np.exp(-tau/tautrip)

    # Neff is not the actual particle number. This formula just looks nicer
    # this way.
    # What would be easier to get is:
    # 1 / (Conc * deva * np.pi * r0) * gz * g2D

    return 1 / (Neff) * g2D * gz * triplet



def MoreInfo_6013(parms, countrate):
    u"""Supplementary variables:
        Beware that the effective volume is chosen arbitrarily.
        Correlation function at lag time τ=0:
        [4] G(τ=0)
        Effective detection volume:         
        [5] V_eff  = π * r₀² * d_eva
        Effective particle concentration:
        [6] C_3D [nM] = C_3D [1000/µm³] * 10000/6.0221415
    """
    #D = parms[0]
    r0 = parms[1]
    deva = parms[2]
    Conc = parms[3]
    Info=list()
    # Detection area:
    Veff = np.pi * r0**2 * deva
    Neff = Conc * Veff
    # Correlation function at tau = 0
    G_0 = CF_Gxyz_TIR_gauss(parms, 0)
    Info.append(["G(0)", G_0])
    Info.append(["V_eff [al]", Veff])
    Info.append(["C_3D [nM]", Conc * 10000/6.0221415])
    if countrate is not None:
        # CPP
        cpp = countrate/Neff
        Info.append(["cpp [kHz]", cpp])
    return Info


def MoreInfo_6014(parms, countrate):
    u"""Supplementary variables:
        Beware that the effective volume is chosen arbitrarily.
        Correlation function at lag time τ=0:
        [6] G(τ=0)
        Effective detection volume:         
        [7] V_eff  = π * r₀² * d_eva
        Effective particle concentration:
        [8] C_3D [nM] = C_3D [1000/µm³] * 10000/6.0221415
    """
    #D = parms[0]
    r0 = parms[1]
    deva = parms[2]
    Conc = parms[3]
    Info=list()
    # Detection area:
    Veff = np.pi * r0**2 * deva
    Neff = Conc * Veff
    # Correlation function at tau = 0
    G_0 = CF_Gxyz_TIR_gauss(parms, 0)
    Info.append(["G(0)", G_0])
    Info.append(["V_eff [al]", Veff])
    Info.append(["C_3D [nM]", Conc * 10000/6.0221415])
    if countrate is not None:
        # CPP
        cpp = countrate/Neff
        Info.append(["cpp [kHz]", cpp])
    return Info


# 3D Model TIR gaussian
m_3dtirsq6013 = [6013, "3D","Simple 3D diffusion w/ TIR",
                 CF_Gxyz_TIR_gauss]
labels_6013 = [u"D [10 µm²/s]",
               u"r₀ [100 nm]",
               u"d_eva [100 nm]",
               u"C_3D [1000/µm³)"]
values_6013 = [2.5420,
               9.44,
               1.0,
               0.03011]
# For user comfort we add values that are human readable.
# Theese will be used for output that only humans can read.
labels_human_readable_6013 = [u"D [µm²/s]",
                              u"r₀ [nm]",
                              u"d_eva [nm]",
                              u"C_3D [1/µm³]"]
values_factor_human_readable_6013 = [10, 
                                     100,
                                     100,
                                     1000]
valuestofit_6013 = [True, False, False, True]
parms_6013 = [labels_6013, values_6013, valuestofit_6013,
          labels_human_readable_6013, values_factor_human_readable_6013]

# Pack the models
model1 = dict()
model1["Parameters"] = parms_6013
model1["Definitions"] = m_3dtirsq6013
model1["Supplements"] = MoreInfo_6013
model1["Verification"] = lambda parms: np.abs(parms)


# 3D Model TIR gaussian + triplet
m_3dtirsq6014 = [6014, "T+3D","Simple 3D diffusion + triplet w/ TIR",
                 CF_Gxyz_TIR_gauss_trip]
labels_6014 = [u"D [10 µm²/s]",
               u"r₀ [100 nm]",
               u"d_eva [100 nm]",
               u"C_3D [1000/µm³)",
               u"τ_trip [ms]",
               u"T"]
values_6014 = [2.5420,
               9.44,
               1.0,
               0.03011,
               0.001,
               0.01]
labels_human_readable_6014 = [u"D [µm²/s]",
                              u"r₀ [nm]",
                              u"d_eva [nm]",
                              u"C_3D [1/µm³]",
                              u"τ_trip [µs]",
                              u"T"]
values_factor_human_readable_6014 = [10,
                                     100,
                                     100,
                                     1000,
                                     1000,
                                     1]
valuestofit_6014 = [True, False, False, True, False, False]
parms_6014 = [labels_6014, values_6014, valuestofit_6014,
          labels_human_readable_6014, values_factor_human_readable_6014]

# Pack the models
model2 = dict()
model2["Parameters"] = parms_6014
model2["Definitions"] = m_3dtirsq6014
model2["Supplements"] = MoreInfo_6014
model2["Verification"] = lambda parms: np.abs(parms)


Modelarray = [model1, model2]



# -*- coding: utf-8 -*-
""" PyCorrFit - Module openfile

This file contains definitions for opening PyCorrFit sessions and
saving PyCorrFit correlation curves.

Dimensionless representation:
unit of time        : 1 ms
unit of inverse time: 10³ /s
unit of distance    : 100 nm
unit of Diff.coeff  : 10 µm²/s
unit of inverse area: 100 /µm²
unit of inv. volume : 1000 /µm³

Copyright (C) 2011-2012  Paul Müller

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""


import csv
import numpy as np
import os
import shutil
import tempfile
import yaml
import zipfile
import warnings

from . import doc

# These imports are required for loading data
from .readfiles import Filetypes
from .readfiles import BGFiletypes



def LoadSessionData(sessionfile, parameters_only=False):
    """ Load PyCorrFit session data from a zip file (.pcfs)


    Parameters
    ----------
    sessionfile : str
        File from which data will be loaded
    parameters_only : bool
        Only load the parameters from the YAML file
    
    
    Returns
    -------
    Infodict : dict
        Infodict may contain the following keys:
        "Backgrounds", list: contains the backgrounds
        "Comments", dict: "Session" comment and int keys to Page titles
        "Correlations", dict: page numbers, all correlation curves
        "External Functions", dict: modelids to external model functions
        "External Weights", dict: page numbers, external weights for fitting
        "Parameters", dict: page numbers, all parameters of the pages
        "Preferences", dict: not used yet
        "Traces", dict: page numbers, all traces of the pages
        "Version", str: the PyCorrFit version of the session
    """
    Infodict = dict()
    # Get the version
    Arc = zipfile.ZipFile(sessionfile, mode='r')
    readmefile = Arc.open("Readme.txt")
    # e.g. "This file was created using PyCorrFit version 0.7.6"
    Infodict["Version"] = readmefile.readline()[46:].strip()
    readmefile.close()
    # Get the yaml parms dump:
    yamlfile = Arc.open("Parameters.yaml")
    # Parameters: Fitting and drawing parameters of correlation curve
    # The *yamlfile* is responsible for the order of the Pages #i.
    Infodict["Parameters"] = yaml.safe_load(yamlfile)
    yamlfile.close()
    if parameters_only:
        Arc.close()
        return Infodict
    # Supplementary data (errors of fit)
    supname = "Supplements.yaml"
    try:
        Arc.getinfo(supname)
    except:
        pass
    else:
        supfile = Arc.open(supname)
        supdata = yaml.safe_load(supfile)
        Infodict["Supplements"] = dict()
        for idp in supdata:
            Infodict["Supplements"][idp[0]] = dict() 
            Infodict["Supplements"][idp[0]]["FitErr"] = idp[1]
            if len(idp) > 2:
                # As of version 0.7.4 we save chi2 and shared pages -global fit
                Infodict["Supplements"][idp[0]]["Chi sq"] = idp[2]
                Infodict["Supplements"][idp[0]]["Global Share"] = idp[3]
    ## Preferences: Reserved for a future version of PyCorrFit :)
    prefname = "Preferences.yaml"
    try:
        Arc.getinfo(prefname)
    except KeyError:
        pass
    else:
        yamlpref = Arc.open(prefname)
        Infodict["Preferences"] = yaml.safe_load(yamlpref)
        yamlpref.close()
    # Get external functions
    Infodict["External Functions"] = dict()
    key = 7001
    while key <= 7999:
        # (There should not be more than 1000 functions)
        funcfilename = "model_"+str(key)+".txt"
        try:
            Arc.getinfo(funcfilename)
        except KeyError:
            # No more functions to import
            key = 8000
        else:
            funcfile =  Arc.open(funcfilename)
            Infodict["External Functions"][key] = funcfile.read()
            funcfile.close()
            key=key+1
    # Get the correlation arrays
    Infodict["Correlations"] = dict()
    for i in np.arange(len(Infodict["Parameters"])):
        # The *number* is used to identify the correct file
        number = str(Infodict["Parameters"][i][0]).strip().strip(":").strip("#")
        pageid = int(number)
        expfilename = "data"+number+".csv"
        expfile = Arc.open(expfilename, 'r')
        readdata = csv.reader(expfile, delimiter=',')
        dataexp = list()
        tau = list()
        if str(readdata.next()[0]) == "# tau only":
            for row in readdata:
                # Exclude commentaries
                if (str(row[0])[0:1] != '#'):
                    tau.append(float(row[0]))
            tau = np.array(tau)
            dataexp = None
        else:
            for row in readdata:
                # Exclude commentaries
                if (str(row[0])[0:1] != '#'):
                    dataexp.append((float(row[0]), float(row[1])))
            dataexp = np.array(dataexp)
            tau = dataexp[:,0]
        Infodict["Correlations"][pageid] = [tau, dataexp]
        del readdata
        expfile.close()
    # Get the Traces
    Infodict["Traces"] = dict()
    for i in np.arange(len(Infodict["Parameters"])):
        # The *number* is used to identify the correct file
        number = str(Infodict["Parameters"][i][0]).strip().strip(":").strip("#")
        pageid = int(number)
        # Find out, if we have a cross correlation data type
        IsCross = False
        try:
            IsCross = Infodict["Parameters"][i][7]
        except IndexError:
            # No Cross correlation
            pass
        if IsCross is False:
            tracefilenames = ["trace"+number+".csv"]
        else:
            # Cross correlation uses two traces
            tracefilenames = ["trace"+number+"A.csv",
                              "trace"+number+"B.csv" ]
        thistrace = list()
        for tracefilename in tracefilenames:
            try:
                Arc.getinfo(tracefilename)
            except KeyError:
                pass
            else:
                tracefile = Arc.open(tracefilename, 'r')
                traceread = csv.reader(tracefile, delimiter=',')
                singletrace = list()
                for row in traceread:
                    # Exclude commentaries
                    if (str(row[0])[0:1] != '#'):
                        singletrace.append((float(row[0]), float(row[1])))
                singletrace = np.array(singletrace)
                thistrace.append(singletrace)
                del traceread
                del singletrace
                tracefile.close()
        if len(thistrace) != 0:
            Infodict["Traces"][pageid] = thistrace
        else:
            Infodict["Traces"][pageid] = None
    # Get the comments, if they exist
    commentfilename = "comments.txt"
    try:
        # Raises KeyError, if file is not present:
        Arc.getinfo(commentfilename)
    except KeyError:
        pass   
    else:
        # Open the file
        commentfile = Arc.open(commentfilename, 'r')
        Infodict["Comments"] = dict()
        for i in np.arange(len(Infodict["Parameters"])):
            number = str(Infodict["Parameters"][i][0]).strip().strip(":").strip("#")
            pageid = int(number)
            # Strip line ending characters for all the Pages.
            Infodict["Comments"][pageid] = commentfile.readline().strip()
        # Now Add the Session Comment (the rest of the file). 
        ComList = commentfile.readlines()
        Infodict["Comments"]["Session"] = ''
        for line in ComList:
            Infodict["Comments"]["Session"] += line
        commentfile.close()
    # Get the Backgroundtraces and data if they exist
    bgfilename = "backgrounds.csv"
    try:
        # Raises KeyError, if file is not present:
        Arc.getinfo(bgfilename)
    except KeyError:
        pass
    else:
        # Open the file
        Infodict["Backgrounds"] = list()
        bgfile = Arc.open(bgfilename, 'r')
        bgread = csv.reader(bgfile, delimiter='\t')
        i = 0
        for bgrow in bgread:
            bgtracefilename = "bg_trace"+str(i)+".csv"
            bgtracefile = Arc.open(bgtracefilename, 'r')
            bgtraceread = csv.reader(bgtracefile, delimiter=',')
            bgtrace = list()
            for row in bgtraceread:
                # Exclude commentaries
                if (str(row[0])[0:1] != '#'):
                    bgtrace.append((np.float(row[0]), np.float(row[1])))
            bgtrace = np.array(bgtrace)
            Infodict["Backgrounds"].append([np.float(bgrow[0]), str(bgrow[1]), bgtrace])
            i = i + 1
        bgfile.close()
    # Get external weights if they exist
    WeightsFilename = "externalweights.txt"
    try:
        # Raises KeyError, if file is not present:
        Arc.getinfo(WeightsFilename)
    except:
        pass
    else:
        Wfile = Arc.open(WeightsFilename, 'r')
        Wread = csv.reader(Wfile, delimiter='\t')
        Weightsdict = dict()
        for wrow in Wread:
            Pkey = wrow[0]  # Page of weights
            pageid = int(Pkey)
            # Do not overwrite anything
            try:
                Weightsdict[pageid]
            except:
                Weightsdict[pageid] = dict()
            Nkey = wrow[1]  # Name of weights
            Wdatafilename = "externalweights_data"+Pkey+"_"+Nkey+".csv"
            Wdatafile = Arc.open(Wdatafilename, 'r')
            Wdatareader = csv.reader(Wdatafile)
            Wdata = list()
            for row in Wdatareader:
                # Exclude commentaries
                if (str(row[0])[0:1] != '#'):
                    Wdata.append(np.float(row[0]))
            Weightsdict[pageid][Nkey] = np.array(Wdata)
        Infodict["External Weights"] = Weightsdict
    Arc.close()
    return Infodict


def SaveSessionData(sessionfile, Infodict):
    """ Session PyCorrFit session data to file.
    
    
    Parameters
    ----------
    sessionfile : str
        The suffix ".pcfs" is automatically appended.
    Infodict : dict
        Infodict may contain the following keys:
        "Backgrounds", list: contains the backgrounds
        "Comments", dict: "Session" comment and int keys to Page titles
        "Correlations", dict: page numbers, all correlation curves
        "External Functions, dict": modelids to external model functions
        "External Weights", dict: page numbers, external weights for fitting
        "Parameters", dict: page numbers, all parameters of the pages
        "Preferences", dict: not used yet
        "Traces", dict: page numbers, all traces of the pages


    The version of PyCorrFit is written to Readme.txt
    """
    (dirname, filename) = os.path.split(sessionfile)
    # Sometimes you have multiple endings...
    if filename.endswith(".pcfs") is not True:
        filename += ".pcfs"
    # Change working directory
    returnWD = os.getcwd()
    tempdir = tempfile.mkdtemp()
    os.chdir(tempdir)
    # Create zip file
    Arc = zipfile.ZipFile(filename, mode='w')
    # Only do the Yaml thing for safe operations.
    # Make the yaml dump
    parmsfilename = "Parameters.yaml"
    # Parameters have to be floats in lists
    # in order for yaml.safe_load to work.
    Parms =  Infodict["Parameters"]
    ParmsKeys = Parms.keys()
    ParmsKeys.sort()
    Parmlist = list()
    for idparm in ParmsKeys:
        # Make sure we do not accidently save arrays.
        # This would not work correctly with yaml.
        Parms[idparm][2] = np.array(Parms[idparm][2],dtype="float").tolist()
        Parms[idparm][3] = np.array(Parms[idparm][3],dtype="bool").tolist()
        # Range of fitting parameters
        Parms[idparm][9] = np.array(Parms[idparm][9],dtype="float").tolist()
        Parmlist.append(Parms[idparm])
    yaml.dump(Parmlist, open(parmsfilename, "wb"))
    Arc.write(parmsfilename)
    os.remove(os.path.join(tempdir, parmsfilename))
    # Supplementary data (errors of fit)
    errsfilename = "Supplements.yaml"
    Sups =  Infodict["Supplements"]
    SupKeys = Sups.keys()
    SupKeys.sort()
    Suplist = list()
    for idsup in SupKeys:
        error = Sups[idsup]["FitErr"]
        chi2 = Sups[idsup]["Chi sq"]
        globalshare = Sups[idsup]["Global Share"]
        Suplist.append([idsup, error, chi2, globalshare])
    yaml.dump(Suplist, open(errsfilename, "wb"))
    Arc.write(errsfilename)
    os.remove(os.path.join(tempdir, errsfilename))
    # Save external functions
    for key in Infodict["External Functions"].keys():
        funcfilename = "model_"+str(key)+".txt"
        funcfile =  open(funcfilename, 'wb')
        funcfile.write(Infodict["External Functions"][key])
        funcfile.close()
        Arc.write(funcfilename)
        os.remove(os.path.join(tempdir, funcfilename))
    # Save (dataexp and tau)s into separate csv files.
    for pageid in Infodict["Correlations"].keys():
        # Since *Array* and *Parms* are in the same order (the page order),
        # we will identify the filename by the Page title number.
        number = str(pageid)
        expfilename = "data"+number+".csv"
        expfile = open(expfilename, 'wb')
        tau = Infodict["Correlations"][pageid][0]
        exp = Infodict["Correlations"][pageid][1]
        dataWriter = csv.writer(expfile, delimiter=',')
        if exp is not None:
            # Names of Columns
            dataWriter.writerow(['# tau', 'experimental data'])
            # Actual Data
            # Do not use len(tau) instead of len(exp[:,0])) !
            # Otherwise, the experimental data will not be saved entirely,
            # if it has been cropped. Because tau might be smaller, than
            # exp[:,0] --> tau = exp[startcrop:endcrop,0]
            for j in np.arange(len(exp[:,0])):
                dataWriter.writerow(["%.20e" % exp[j,0],
                                     "%.20e" % exp[j,1]])
        else:
            # Only write tau
            dataWriter.writerow(['# tau'+' only'])
            for j in np.arange(len(tau)):
                dataWriter.writerow(["%.20e" % tau[j]])
        expfile.close()
        # Add to archive
        Arc.write(expfilename)
        os.remove(os.path.join(tempdir, expfilename))
    # Save traces into separate csv files.
    for pageid in Infodict["Traces"].keys():
        number = str(pageid)
        # Since *Trace* and *Parms* are in the same order, which is the
        # Page order, we will identify the filename by the Page title 
        # number.
        if Infodict["Traces"][pageid] is not None:
            if Parms[pageid][7] is True:
                # We have cross correlation: save two traces
                ## A
                tracefilenamea = "trace"+number+"A.csv"
                tracefile = open(tracefilenamea, 'wb')
                traceWriter = csv.writer(tracefile, delimiter=',')
                time = Infodict["Traces"][pageid][0][:,0]
                rate = Infodict["Traces"][pageid][0][:,1]
                # Names of Columns
                traceWriter.writerow(['# time', 'count rate'])
                # Actual Data
                for j in np.arange(len(time)):
                    traceWriter.writerow(["%.20e" % time[j],
                                          "%.20e" % rate[j]])
                tracefile.close()
                # Add to archive
                Arc.write(tracefilenamea)
                os.remove(os.path.join(tempdir, tracefilenamea))
                ## B
                tracefilenameb = "trace"+number+"B.csv"
                tracefile = open(tracefilenameb, 'wb')
                traceWriter = csv.writer(tracefile, delimiter=',')
                time = Infodict["Traces"][pageid][1][:,0]
                rate = Infodict["Traces"][pageid][1][:,1]
                # Names of Columns
                traceWriter.writerow(['# time', 'count rate'])
                # Actual Data
                for j in np.arange(len(time)):
                    traceWriter.writerow(["%.20e" % time[j],
                                          "%.20e" % rate[j]])
                tracefile.close()
                # Add to archive
                Arc.write(tracefilenameb)
                os.remove(os.path.join(tempdir, tracefilenameb))
            else:
                # Save one single trace
                tracefilename = "trace"+number+".csv"
                tracefile = open(tracefilename, 'wb')
                traceWriter = csv.writer(tracefile, delimiter=',')
                time = Infodict["Traces"][pageid][:,0]
                rate = Infodict["Traces"][pageid][:,1]
                # Names of Columns
                traceWriter.writerow(['# time', 'count rate'])
                # Actual Data
                for j in np.arange(len(time)):
                    traceWriter.writerow(["%.20e" % time[j],
                                          "%.20e" % rate[j]])
                tracefile.close()
                # Add to archive
                Arc.write(tracefilename)
                os.remove(os.path.join(tempdir, tracefilename))
    # Save comments into txt file
    commentfilename = "comments.txt"
    commentfile = open(commentfilename, 'wb')
    # Comments[-1] is comment on whole Session
    Ckeys = Infodict["Comments"].keys()
    Ckeys.sort()
    for key in Ckeys:
        if key != "Session":
            commentfile.write(Infodict["Comments"][key]+"\r\n")
    commentfile.write(Infodict["Comments"]["Session"])
    commentfile.close()
    Arc.write(commentfilename)
    os.remove(os.path.join(tempdir, commentfilename))
    ## Save Background information:
    Background = Infodict["Backgrounds"]
    if len(Background) > 0:
        # We do not use a comma separated, but a tab separated file,
        # because a comma might be in the name of a bg.
        bgfilename = "backgrounds.csv"
        bgfile = open(bgfilename, 'wb')
        bgwriter = csv.writer(bgfile, delimiter='\t')
        for i in np.arange(len(Background)):
            bgwriter.writerow([str(Background[i][0]), Background[i][1]])
            # Traces
            bgtracefilename = "bg_trace"+str(i)+".csv"
            bgtracefile = open(bgtracefilename, 'wb')
            bgtraceWriter = csv.writer(bgtracefile, delimiter=',')
            bgtraceWriter.writerow(['# time', 'count rate'])
            # Actual Data
            time = Background[i][2][:,0]
            rate = Background[i][2][:,1]
            for j in np.arange(len(time)):
                bgtraceWriter.writerow(["%.20e" % time[j],
                                        "%.20e" % rate[j]])
            bgtracefile.close()
            # Add to archive
            Arc.write(bgtracefilename)
            os.remove(os.path.join(tempdir, bgtracefilename))
        bgfile.close()
        Arc.write(bgfilename)
        os.remove(os.path.join(tempdir, bgfilename))
    ## Save External Weights information
    WeightedPageID = Infodict["External Weights"].keys()
    WeightedPageID.sort()
    WeightFilename = "externalweights.txt"
    WeightFile = open(WeightFilename, 'wb')
    WeightWriter = csv.writer(WeightFile, delimiter='\t')
    for pageid in WeightedPageID:
        number = str(pageid)
        NestWeights = Infodict["External Weights"][pageid].keys()
        # The order of the types does not matter, since they are
        # sorted in the frontend and upon import. We sort them here, anyhow.
        NestWeights.sort()
        for Nkey in NestWeights:
            WeightWriter.writerow([number, str(Nkey).strip()])
            # Add data to a File
            WeightDataFilename = "externalweights_data"+number+\
                                 "_"+str(Nkey).strip()+".csv"
            WeightDataFile = open(WeightDataFilename, 'wb')
            WeightDataWriter = csv.writer(WeightDataFile)
            wdata = Infodict["External Weights"][pageid][Nkey]
            for jw in np.arange(len(wdata)):
                WeightDataWriter.writerow([str(wdata[jw])])
            WeightDataFile.close()
            Arc.write(WeightDataFilename)
            os.remove(os.path.join(tempdir, WeightDataFilename))
    WeightFile.close()
    Arc.write(WeightFilename)
    os.remove(os.path.join(tempdir, WeightFilename))
    ## Readme
    rmfilename = "Readme.txt"
    rmfile = open(rmfilename, 'wb')
    rmfile.write(ReadmeSession)
    rmfile.close()
    Arc.write(rmfilename)
    os.remove(os.path.join(tempdir, rmfilename))
    # Close the archive
    Arc.close()
    # Move archive to destination directory
    shutil.move(os.path.join(tempdir, filename), 
                os.path.join(dirname, filename) )
    # Go to destination directory
    os.chdir(returnWD)
    os.rmdir(tempdir)


def ExportCorrelation(exportfile, Page, info, savetrace=True):
    """ Write correlation data to a file
        
    
    Parameters
    ----------
    exportfile : str
        Absolute filename to save data
    Page : PyCorrFit Page object
        Contains all correlation data
    info : module
        The `info` tool module. This is a workaround until Page has
        its own class to create info data.
    savetrace : bool
        Append the trace to the file
    """

    openedfile = open(exportfile, 'wb')
    ## First, some doc text
    openedfile.write(ReadmeCSV.replace('\n', '\r\n'))
    # The infos
    InfoMan = info.InfoClass(CurPage=Page)
    PageInfo = InfoMan.GetCurFancyInfo()
    for line in PageInfo.splitlines():
        openedfile.write("# "+line+"\r\n")
    openedfile.write("#\r\n#\r\n")
    # Get all the data we need from the Page
    # Modeled data
    # Since 0.7.8 the user may normalize the curves. The normalization
    # factor is set in *Page.normfactor*.
    corr = Page.datacorr[:,1]*Page.normfactor
    if Page.dataexp is not None:
        # Experimental data
        tau = Page.dataexp[:,0]
        exp = Page.dataexp[:,1]*Page.normfactor
        res = Page.resid[:,1]*Page.normfactor
        # Plotting! Because we only export plotted area.
        weight = Page.weights_used_for_plotting
        if weight is None:
            pass
        elif len(weight) != len(exp):
            text = "Weights have not been calculated for the "+\
                   "area you want to export. Pressing 'Fit' "+\
                   "again should solve this issue. Weights will "+\
                   "not be saved."
            warnings.warn(text)
            weight = None
    else:
        tau = Page.datacorr[:,0]
        exp = None
        res = None
    # Include weights in data saving:
    # PyCorrFit thinks in [ms], but we will save as [s]
    timefactor = 0.001
    tau = timefactor * tau
    ## Now we want to write all that data into the file
    # This is for csv writing:
    ## Correlation curve
    dataWriter = csv.writer(openedfile, delimiter='\t')
    if exp is not None:
        header = '# Channel (tau [s])'+"\t"+ \
                 'Experimental correlation'+"\t"+ \
                 'Fitted correlation'+ "\t"+ \
                 'Residuals'+"\r\n"
        data = [tau, exp, corr, res]
        if Page.weighted_fit_was_performed is True \
        and weight is not None:
            header = header.strip() + "\t"+'Weights (fit)'+"\r\n"
            data.append(weight)
    else:
        header = '# Channel (tau [s])'+"\t"+ \
                 'Correlation function'+"\r\n"
        data = [tau, corr]
    # Write header
    openedfile.write(header)
    # Write data
    for i in np.arange(len(data[0])):
        # row-wise, data may have more than two elements per row
        datarow = list()
        for j in np.arange(len(data)):
            rowcoli = str("%.10e") % data[j][i]
            datarow.append(rowcoli)
        dataWriter.writerow(datarow)
    ## Trace
    # Only save the trace if user wants us to:
    if savetrace:
        # We will also save the trace in [s]
        # Intensity trace in kHz may stay the same
        if Page.trace is not None:
            # Mark beginning of Trace
            openedfile.write('#\r\n#\r\n# BEGIN TRACE\r\n#\r\n')
            # Columns
            time = Page.trace[:,0]*timefactor
            intensity = Page.trace[:,1]
            # Write
            openedfile.write('# Time [s]'+"\t" 
                                 'Intensity trace [kHz]'+" \r\n")
            for i in np.arange(len(time)):
                dataWriter.writerow([str("%.10e") % time[i],
                                     str("%.10e") % intensity[i]])
        elif Page.tracecc is not None:
            # We have some cross-correlation here:
            # Mark beginning of Trace A
            openedfile.write('#\r\n#\r\n# BEGIN TRACE\r\n#\r\n')
            # Columns
            time = Page.tracecc[0][:,0]*timefactor
            intensity = Page.tracecc[0][:,1]
            # Write
            openedfile.write('# Time [s]'+"\t" 
                                 'Intensity trace [kHz]'+" \r\n")
            for i in np.arange(len(time)):
                dataWriter.writerow([str("%.10e") % time[i],
                                     str("%.10e") % intensity[i]])
            # Mark beginning of Trace B
            openedfile.write('#\r\n#\r\n# BEGIN SECOND TRACE\r\n#\r\n')
            # Columns
            time = Page.tracecc[1][:,0]*timefactor
            intensity = Page.tracecc[1][:,1]
            # Write
            openedfile.write('# Time [s]'+"\t" 
                                 'Intensity trace [kHz]'+" \r\n")
            for i in np.arange(len(time)):
                dataWriter.writerow([str("%.10e") % time[i],
                                     str("%.10e") % intensity[i]])

        openedfile.close()


session_wildcards = [".pcfs", ".pycorrfit-session.zip"]


ReadmeCSV = """# This file was created using PyCorrFit version {}.
#
# Lines starting with a '#' are treated as comments.
# The data is stored as CSV below this comment section.
# Data usually consists of lag times (channels) and
# the corresponding correlation function - experimental
# and fitted values plus resulting residuals.
# If this file is opened by PyCorrFit, only the first two
# columns will be imported as experimental data.
#
""".format(doc.__version__)
    
    
ReadmeSession = """This file was created using PyCorrFit version {}.
The .zip archive you are looking at is a stored session of PyCorrFit.
If you are interested in how the data is stored, you will find
out here. Most important are the dimensions of units:
Dimensionless representation:
 unit of time        : 1 ms
 unit of inverse time: 10³ /s
 unit of distance    : 100 nm
 unit of Diff.coeff  : 10 µm²/s
 unit of inverse area: 100 /µm²
 unit of inv. volume : 1000 /µm³
From there, the dimension of any parameter may be
calculated.

There are a number of files within this archive, 
depending on what was done during the session.

backgrounds.csv
 - Contains the list of backgrounds used and
 - Averaged intensities in [kHz]

bg_trace*.csv (where * is an integer)
 - The trace of the background corresponding
   to the line number in backgrounds.csv
 - Time in [ms], Trace in [kHz]

comments.txt
 - Contains page titles and session comment
 - First n lines are titles, rest is session
   comment (where n is total number of pages)

data*.csv (where * is (Number of page))
 - Contains lag times [ms]
 - Contains experimental data, if available

externalweights.txt
 - Contains names (types) of external weights other than from
   Model function or spline fit
 - Linewise: 1st element is page number, 2nd is name
 - According to this data, the following files are present in the archive

externalweights_data_*PageID*_*Type*.csv
 - Contains weighting information of Page *PageID* of type *Type*

model_*ModelID*.txt
 - An external (user-defined) model file with internal ID *ModelID*

Parameters.yaml
 - Contains all Parameters for each page
   Block format:
    - - '#(Number of page): '       
      - (Internal model ID)
      - (List of parameters)
      - (List of checked parameters (for fitting))
      - [(Min channel selected), (Max channel selected)]
      - [(Weighted fit method (0=None, 1=Spline, 2=Model function)), 
         (No. of bins from left and right),
         (No. of knots (of e.g. spline)),
         (Type of fitting algorithm (e.g. "Lev-Mar", "Nelder-Mead")]
      - [B1,B2] Background to use (line in backgrounds.csv)
           B2 is always *null* for autocorrelation curves
      - Data type is Cross-correlation?
      - Parameter id (int) used for normalization in plotting.
        This number first enumerates the model parameters and then
        the supplemental parameters (e.g. "n1").
      - - [min, max] fitting parameter range of 1st parameter
        - [min, max] fitting parameter range of 2nd parameter
        - etc.
 - Order in Parameters.yaml defines order of pages in a session
 - Order in Parameters.yaml defines order in comments.txt

Readme.txt (this file)

Supplements.yaml
 - Contains errors of fitting
   Format:
   -- Page number
    -- [parameter id, error value]
     - [parameter id, error value]
    - Chi squared
    - [pages that share parameters] (from global fitting)
     
trace*.csv (where * is (Number of page) | appendix "A" or "B" point to
            the respective channels (only in cross-correlation mode))
 - Contains times [ms]
 - Contains countrates [kHz]
""".format(doc.__version__)



# -*- coding: utf-8 -*-
""" PyCorrFit

    Module misc

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

import codecs
from distutils.version import LooseVersion # For version checking
import numpy as np
import os
import sys
import tempfile
import urllib2
import webbrowser
import wx                               # GUI interface wxPython
import wx.html
import wx.lib.delayedresult as delayedresult

from . import doc                          # Documentation/some texts
# The icon file was created with
# img2py -i -n Main PyCorrFit_icon.png icon.py
from . import icon                         # Contains the program icon


class UpdateDlg(wx.Frame):
    def __init__(self, parent, valuedict):
        
        description = valuedict["Description"]
        homepage = valuedict["Homepage"]
        githome = valuedict["Homepage_GIT"]
        changelog = valuedict["Changelog"]
        pos = parent.GetPosition()
        pos = (pos[0]+100, pos[1]+100)
        wx.Frame.__init__(self, parent, wx.ID_ANY, title="Update", 
                          size=(250,180), pos=pos)
        self.changelog = changelog
        # Fill html content
        html = wxHTML(self)
        string =             '' +\
            "<b> PyCorrFit <br></b>" +\
            "Your version: " + description[0]+"<br>" +\
            "Latest version: " + description[1]+"<br>" +\
            "(" + description[2]+")<br><p><b>"
        if len(homepage) != 0:
            string = string + '<a href="'+homepage+'">Homepage</a><br>'
        if len(githome) != 0:
            string = string + '<a href="'+githome+'">Repository</a><br>'

        if len(changelog) != 0:
            string = string + \
                     '<a href="'+changelog+'">Change Log</a>'
        string = string+'</b></p>'
        html.SetPage(string)
        self.Bind(wx.EVT_CLOSE, self.Close)
        # Set window icon
        ico = getMainIcon()
        wx.Frame.SetIcon(self, ico)


    def Close(self, event):
        if len(self.changelog) != 0:
            # Cleanup downloaded file, if it was downloaded
            if self.changelog != doc.StaticChangeLog:
                os.remove(self.changelog)
        self.Destroy()


class wxHTML(wx.html.HtmlWindow):
    def OnLinkClicked(parent, link):
        webbrowser.open(link.GetHref())


def parseString2Pagenum(parent, string, nodialog=False):
    """ Parse a string with a list of pagenumbers to an integer list with
        page numbers.
        e.g. "1-3,5,7" --> [1,2,3,5,7]
        parent is important
    """
    listFull = string.split(",")
    PageNumbers = list()
    try:
        for item in listFull:
            pagerange = item.split("-")
            start = pagerange[0].strip()
            start = int(filter(type(start).isdigit, start))
            end = pagerange[-1].strip()
            end = int(filter(type(end).isdigit, end))
            for i in np.arange(end-start+1)+start:
                PageNumbers.append(i)
        PageNumbers.sort()
        return PageNumbers
    except:
        if nodialog is False:
            errstring = "Invalid syntax in page selection: "+string+\
                        ". Please use a comma separated list with"+\
                        " optional dashes, e.g. '1-3,6,8'." 
            try:
                wx.MessageDialog(parent, errstring, "Error", 
                                  style=wx.ICON_ERROR|wx.OK|wx.STAY_ON_TOP)
            except:
                raise ValueError(errstring)
        else:
            raise ValueError(errstring)
        return None
        

def parsePagenum2String(pagenumlist):
    """ Make a string with dashes and commas from a list of pagenumbers.
        e.g. [1,2,3,5,7] --> "1-3,5,7"
    """
    if len(pagenumlist) == 0:
        return ""
    # Make sure we have integers
    newlist = list()
    for num in pagenumlist:
        newlist.append(int(num))
    newlist.sort()
    # begin string
    string = str(newlist[0])
    # iteration through list:
    dash = False
    for i in np.arange(len(newlist)-1)+1:
        if dash == True:
            if newlist[i]-1 == newlist[i-1]:
                pass
            else:
                string += "-"+str(newlist[i-1])+", "+str(newlist[i])
                dash = False
        else:
            if newlist[i]-1 == newlist[i-1]:
                if newlist[i]-2 == newlist[i-2]:
                    dash = True
                elif len(newlist) != i+1 and newlist[i]+1 == newlist[i+1]:
                    dash = True
                else:
                    string += ", "+str(newlist[i])
                    dash = False
            else:
                dash = False
                string += ", "+str(newlist[i])
        # Put final number
        if newlist[i] == newlist[-1]:
            if parseString2Pagenum(None, string)[-1] != newlist[i]:
                if dash == True:
                    string += "-"+str(newlist[i])
                else:
                    string += ", "+str(newlist[i])
    return string


def removewrongUTF8(name):
    newname = u""
    for char in name:
        try:
            codecs.decode(char, "UTF-8")
        except:
            pass
        else:
            newname += char
    return newname
    

def getMainIcon(pxlength=32):
    """ *pxlength* is the side length in pixels of the icon """
    # Set window icon
    iconBMP = icon.getMainBitmap()
    # scale
    image = wx.ImageFromBitmap(iconBMP)
    image = image.Scale(pxlength, pxlength, wx.IMAGE_QUALITY_HIGH)
    iconBMP = wx.BitmapFromImage(image)
    iconICO = wx.IconFromBitmap(iconBMP)
    return iconICO


def findprogram(program):
    """ Uses the systems PATH variable find executables"""
    path = os.environ['PATH']
    paths = path.split(os.pathsep)
    for d in paths:
        if os.path.isdir(d):
            fullpath = os.path.join(d, program)
            if sys.platform[:3] == 'win':
                for ext in '.exe', '.bat':
                    program_path = fullpath + ext
                    if os.path.isfile(fullpath + ext):
                        return (1, program_path)
            else:
                if os.path.isfile(fullpath):
                    return (1, fullpath)
    return (0, None)


def Update(parent):
    """ This is a thread for _Update """
    parent.StatusBar.SetStatusText("Connecting to server...")
    delayedresult.startWorker(_UpdateConsumer, _UpdateWorker,
                              wargs=(parent,), cargs=(parent,))

def _UpdateConsumer(delayedresult, parent):
    results = delayedresult.get()
    dlg = UpdateDlg(parent, results)
    dlg.Show()
    parent.StatusBar.SetStatusText("...update status: "+results["Description"][2])


def _UpdateWorker(parent):
        changelog = ""
        hpversion = None
        # I created this TXT record to keep track of the current web presence.
        try:
            urlopener = urllib2.urlopen(doc.HomePage, timeout=2)
            homepage = urlopener.geturl()
        except:
            homepage = doc.HomePage
        try:
            urlopener2 = urllib2.urlopen(doc.GitHome, timeout=2)
            githome = urlopener2.geturl()
        except:
            githome = ""
        # Find the changelog file
        try:
            responseCL = urllib2.urlopen(homepage+doc.ChangeLog, timeout=2)
        except:
            CLfile = doc.GitChLog
        else:
            fileresponse = responseCL.read()
            CLlines = fileresponse.splitlines()
            # We have a transition between ChangeLog.txt on the homepage
            # containing the actual changelog or containing a link to
            # the ChangeLog file.
            if len(CLlines) == 1:
                CLfile = CLlines[0]
            else:
                hpversion = CLlines[0]
                CLfile = doc.GitChLog
        # Continue version comparison if True
        continuecomp = False
        try:
            responseVer = urllib2.urlopen(CLfile, timeout=2)
        except:
            if hpversion == None:
                newversion = "unknown"
                action = "cannot connect to server"
            else:
                newversion = hpversion
                continuecomp = True
        else:
            continuecomp = True
            changelog = responseVer.read()
            newversion = changelog.splitlines()[0]
        if continuecomp:
            new = LooseVersion(newversion)
            old = LooseVersion(parent.version)
            if new > old:
                action = "update available"
            elif new < old:
                action = "whoop you rock!"
            else:
                action = "state of the art"
        description = [parent.version, newversion, action]
        if len(changelog) != 0:
            changelogfile = tempfile.mktemp()+"_PyCorrFit_ChangeLog"+".txt"
            clfile = open(changelogfile, 'wb')
            clfile.write(changelog)
            clfile.close()            
        else:
            changelogfile=doc.StaticChangeLog
        results = dict()
        results["Description"] = description
        results["Homepage"] = homepage
        results["Homepage_GIT"] = githome
        results["Changelog"] = changelogfile
        return results



# -*- coding: utf-8 -*-
"""
    When a membrane is scanned perpendicularly to its surface, the
    fluorescence signal originating from the membrane itself must be
    separated from the signal of the surrounding medium for an FCS
    analysis. PyCorrFit interactively extracts the fluctuating
    fluorescence signal from such measurements and applies a
    multiple-tau algorithm. The obtained correlation curves can be
    evaluated using PyCorrFit.

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from . import doc
from . import main

## VERSION
version = doc.__version__
__version__ = version

main.Main()



# -*- coding: utf-8 -*-
"""
    PyCorrFit
    This file contains a 3D+2D+T confocal FCS model.

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""
from __future__ import division

import numpy as np

# 3D + 2D + T
def CF_Gxyz_3d2dT_gauss(parms, tau):
    u""" Two-component, two- and three-dimensional diffusion
        with a Gaussian laser profile, including a triplet component.
        The triplet factor takes into account blinking according to triplet
        states of excited molecules.
        Set *T* or *τ_trip* to 0, if no triplet component is wanted.

        particle2D = (1-F)/ (1+τ/τ_2D) 
        particle3D = α*F/( (1+τ/τ_3D) * sqrt(1+τ/(τ_3D*SP²)))
        triplet = 1 + T/(1-T)*exp(-τ/τ_trip)
        norm = (1-F + α*F)²
        G = 1/n*(particle1 + particle2)*triplet/norm + offset

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] n       Effective number of particles in confocal volume
                    (n = n2D+n3D)
        [1] τ_2D    Diffusion time of surface bound particls
        [2] τ_3D    Diffusion time of freely diffusing particles
        [3] F       Fraction of molecules of the freely diffusing species
                    (n3D = n*F), 0 <= F <= 1
        [4] SP      SP=z₀/r₀ Structural parameter,
                         describes elongation of the confocal volume
        [5] α       Relative molecular brightness of particle
                    3D compared to particle 2D (α = q3D/q2D)
        [6] τ_trip  Characteristic residence time in triplet state
        [7] T       Fraction of particles in triplet (non-fluorescent) state
                    0 <= T < 1
        [8] offset
        *tau* - lag time
    """
    n=parms[0]
    taud2D=parms[1]
    taud3D=parms[2]
    F=parms[3]
    SP=parms[4]
    alpha=parms[5]
    tautrip=parms[6]
    T=parms[7]
    off=parms[8]


    particle2D = (1-F)/ (1+tau/taud2D) 
    particle3D = alpha**2*F/( (1+tau/taud3D) * np.sqrt(1+tau/(taud3D*SP**2)))
    if tautrip == 0 or T == 0:
        triplet = 1
    else:
        triplet = 1 + T/(1-T) * np.exp(-tau/tautrip)
    norm = (1-F + alpha*F)**2
    G = 1/n*(particle2D + particle3D)*triplet/norm

    return G + off

def Checkme(parms):
    parms[0] = np.abs(parms[0])
    parms[1] = np.abs(parms[1]) #= taud2D
    parms[2] = np.abs(parms[2]) #= taud3D
    F=parms[3]
    parms[4]=np.abs(parms[4])
    parms[5]=np.abs(parms[5])
    tautrip=np.abs(parms[6])
    T=parms[7]
    #off=parms[8]
    # Triplet fraction is between 0 and one. T may not be one!
    T = (0.<=T<1.)*T + .99999999999999*(T>=1)
    # Fraction of molecules may also be one
    F = (0.<=F<=1.)*F + 1.*(F>1)

    parms[3] = F
    parms[6] = tautrip
    parms[7] = T

    return parms


def MoreInfo(parms, countrate):
    u"""Supplementary parameters:
        Effective number of freely diffusing particles in 3D solution:
        [9]  n3D = n*F
        Effective number particles diffusing on 2D surface:
        [10] n2D = n*(1-F)
    """
    # We can only give you the effective particle number
    n = parms[0]
    F3d = parms[3]
    Info = list()
    # The enumeration of these parameters is very important for
    # plotting the normalized curve. Countrate must come out last!
    Info.append([u"n3D", n*F3d])
    Info.append([u"n2D", n*(1.-F3d)])
    if countrate is not None:
        # CPP
        cpp = countrate/n
        Info.append([u"cpp [kHz]", cpp])
    return Info


# 3D + 3D + T model gauss
m_gauss_3d_2d_t = [6032, u"T+3D+2D",
                            "Separate 3D and 2D diffusion + triplet, Gauß",
                            CF_Gxyz_3d2dT_gauss]
labels  = [ u"n",
            u"τ_2D [ms]",
            u"τ_3D [ms]",
            u"F_3D", 
            u"SP",
            u"\u03b1"+" (q_3D/q_2D)", 
            u"τ_trip [ms]",
            u"T",
            u"offset"
                ]
values = [ 
                25,      # n
                100,     # taud2D
                0.1,     # taud3D
                0.5,     # F3D
                7,       # SP
                1.0,     # alpha
                0.001,   # tautrip
                0.01,    # T
                0.0      # offset
                ]
# For user comfort we add values that are human readable.
# Theese will be used for output that only humans can read.
labels_human_readable  = [  u"n",
                            u"τ_2D [ms]",
                            u"τ_3D [ms]",
                            u"F_3D", 
                            u"SP",
                            u"\u03b1"+" (q_3D/q_2D)", 
                            u"τ_trip [µs]",
                            u"T",
                            u"offset"
                            ]
values_factor_human_readable = [
                          1.,     # "n",
                          1.,     # "τ_2D [ms]",
                          1.,     # "τ_3D [ms]",
                          1.,     # "F_3D", 
                          1.,     # "SP",
                          1.,     # u"\u03b1"+" (q_3D/q_2D)", 
                          1000.,  # "τ_trip [µs]",
                          1.,     # "T",
                          1.      # "offset"
                ]
valuestofit = [True, True, True, True, False, False, False, False, False]
parms = [labels, values, valuestofit,
         labels_human_readable, values_factor_human_readable]


model1 = dict()
model1["Parameters"] = parms
model1["Definitions"] = m_gauss_3d_2d_t
model1["Verification"] = Checkme
model1["Supplements"] = MoreInfo

Modelarray = [model1]



# -*- coding: utf-8 -*-
""" 
    PyCorrFit
    
    functions in this file: *openCSV*

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""
import os
import csv
import numpy as np


def openCSV(dirname, filename):
    """ Read relevant data from a file looking like this:
        [...]
        # Comment
        # Data type: Autocorrelation
        [...]
        1.000000e-006   3.052373e-001
        1.020961e-006   3.052288e-001
        1.042361e-006   3.052201e-001
        1.064209e-006   3.052113e-001
        1.086516e-006   3.052023e-001
        1.109290e-006   3.051931e-001
        [...]
        # BEGIN TRACE
        [...]
        10.852761   31.41818
        12.058624   31.1271
        13.264486   31.27305
        14.470348   31.33442
        15.676211   31.15861
        16.882074   31.08564
        18.087936   31.21335
        [...]

        Data type:
        If Data type is "Cross-correlation", we will try to import
        two traces after "# BEGIN SECOND TRACE"

        1st section:
         First column denotes tau in seconds and the second row the
         correlation signal.
        2nd section:
         First column denotes tau in seconds and the second row the
         intensity trace in kHz.


        Returns:
        1. A list with tuples containing two elements:
           1st: tau in ms
           2nd: corresponding correlation signal
        2. None - usually is the trace, but the trace is not saved in
                  the PyCorrFit .csv format.
        3. A list with one element, indicating, that we are opening only
           one correlation curve.
    """
    # Check if the file is correlation data
    csvfile = open(os.path.join(dirname, filename), 'r')
    firstline = csvfile.readline()
    if firstline.lower().count("this is not correlation data") > 0:
        csvfile.close()
        return None
    csvfile.close()
    
    # Define what will happen to the file
    timefactor = 1000 # because we want ms instead of s
    csvfile = open(os.path.join(dirname, filename), 'r')
    readdata = csv.reader(csvfile, delimiter=',')
    data = list()
    trace = None
    traceA = None
    DataType="AC" # May be changed
    numtraces = 0
    for row in readdata:
        if len(row) == 0 or len(str(row[0]).strip()) == 0:
            # Do nothing with empty/whitespace lines
            pass
            # Beware that the len(row) statement has to be called first
            # (before the len(str(row[0]).strip()) ). Otherwise some
            # error would be raised.
        elif str(row[0])[:12].lower() == "# Type AC/CC".lower():
            corrtype = str(row[0])[12:].strip().strip(":").strip()
            if corrtype[:17].lower() == "cross-correlation":
                # We will later try to import a second trace
                DataType="CC"
                DataType += corrtype[17:].strip()
            elif corrtype[0:15].lower() == "autocorrelation":
                DataType="AC"
                DataType += corrtype[15:].strip()         
        elif str(row[0])[0:13].upper() == '# BEGIN TRACE':
            # Correlation is over. We have a trace
            corr = np.array(data)
            data=list()
            numtraces = 1
        elif str(row[0])[0:20].upper() == '# BEGIN SECOND TRACE':
            # First trace is over. We have a second trace
            traceA = np.array(data)
            data = list()
            numtraces = 2
        # Exclude commentaries
        elif str(row[0])[0:1] != '#':
            # Read the 1st section
            # On Windows we had problems importing nan values that
            # had some white-spaces around them. Therefore: strip()
            ## As of version 0.7.8 we are supporting white space
            ## separated values as well
            if len(row) == 1:
                row = row[0].split()
            data.append((np.float(row[0].strip())*timefactor, 
                         np.float(row[1].strip())))
    # Collect the rest of the trace, if there is any:
    rest = np.array(data)
    if numtraces == 0:
        corr = rest
    elif numtraces >= 1:
        trace = rest
    del data
    ## Remove any NaN numbers from thearray
    # Explanation:
    # np.isnan(data)
    #  finds the position of NaNs in the array (True positions); 2D array, bool
    # any(1)
    #  finds the rows that have True in them; 1D array, bool
    # ~
    #  negates them and is given as an argument (array type bool) to
    #  select which items we want.
    corr = corr[~np.isnan(corr).any(1)]
    # Also check for infinities.
    corr = corr[~np.isinf(corr).any(1)]
    csvfile.close()
    Traces=list()
    # Set correct trace data for import
    if numtraces == 1 and DataType[:2] == "AC":
        Traces.append(trace)
    elif numtraces == 2 and DataType[:2] == "CC":
        Traces.append([traceA, trace])
    elif numtraces == 1 and DataType[:2] == "CC":
        # Should not happen, but for convenience:
        Traces.append([trace, trace])
    else:
        Traces.append(None)
    dictionary = dict()
    dictionary["Correlation"] = [corr]
    dictionary["Trace"] = Traces
    dictionary["Type"] = [DataType]
    dictionary["Filename"] = [filename]
    return dictionary



# -*- coding: utf-8 -*-
"""
    PyCorrFit

    Module models:
    Define all models and set initial parameters.

    Each model has a unique ID. This ID is very important:
        1. It is a wxWidgets ID.
        2. It is used in the saving of sessions to identify a model.
    It is very important, that model IDs do NOT change in newer versions
    of PyCorrFit, because it would not be possible to restore older
    PyCorrFit sessions.

    Dimensionless representation:
    unit of time        : 1 ms
    unit of inverse time: 10³ /s
    unit of distance    : 100 nm
    unit of Diff.coeff  : 10 µm²/s
    unit of inverse area: 100 /µm²
    unit of inv. volume : 1000 /µm³

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""


# This file is necessary for this folder to become a module that can be 
# imported from within Python/PyCorrFit.

import numpy as np                  # NumPy
import sys


## On Windows XP I had problems with the unicode Characters.
# I found this at 
# http://stackoverflow.com/questions/5419/python-unicode-and-the-windows-console
# and it helped:
reload(sys)
sys.setdefaultencoding('utf-8')


## Models
from . import MODEL_classic_gaussian_2D
from . import MODEL_classic_gaussian_3D
from . import MODEL_classic_gaussian_3D2D
from . import MODEL_TIRF_gaussian_1C
from . import MODEL_TIRF_gaussian_3D2D
from . import MODEL_TIRF_gaussian_3D3D
from . import MODEL_TIRF_1C
from . import MODEL_TIRF_2D2D
from . import MODEL_TIRF_3D2D
from . import MODEL_TIRF_3D3D
from . import MODEL_TIRF_3D2Dkin_Ries

def AppendNewModel(Modelarray):
    """ Append a new model from a modelarray. *Modelarray* has to be a list
        whose elements have two items:
        [0] parameters
        [1] some info about the model
        See separate models for more information
    """
    global values
    global valuedict
    global models
    global modeldict
    global verification

    for Model in Modelarray:
        # We can have many models in one model array
        parms = Model["Parameters"]
        texts = Model["Definitions"]
        values.append(parms)
        # model ID is texts[0]
        valuedict[texts[0]] = parms
        models.append(texts)
        modeldict[texts[0]] = texts
        # Suplementary Data might be there
        try:
             supper = Model["Supplements"]
        except KeyError:
            # Nothing to do
            pass
        else:
            supplement[texts[0]] = supper
        # Check functions - check for correct values
        try:
             verify = Model["Verification"]
        except KeyError:
            # Nothing to do. Return empty function, so we do not need to
            # do this try and error thing again.
            verification[texts[0]] = lambda parms: parms
        else:
            verification[texts[0]] = verify


def GetHumanReadableParms(model, parameters):
    """ From a set of parameters that have internal units e.g. [100 nm],
        Calculate the parameters in human readable units e.g. [nm].
        Uses modeldict from this module.
        *model* - an integer ID of a model
        *parameters* - a list/array of parameters 
                       (all parameters of that model)
        Returns:
        New Units, New Parameters
    """
    stdparms = valuedict[model]
    if len(stdparms) == 5:
        # This means we have extra information on the model
        # Return some human readable stuff
        OldParameters = 1.*np.array(parameters)
        Facors = 1.*np.array(stdparms[4])
        NewParameters = 1.*OldParameters*Facors
        NewUnits = stdparms[3]
        return NewUnits, NewParameters
    else:
        # There is no info about human readable stuff, or it is already human
        # readable.
        return stdparms[0], parameters


def GetHumanReadableParameterDict(model, names, parameters):
    """ From a set of parameters that have internal units e.g. [100 nm],
        Calculate the parameters in human readable units e.g. [nm].
        Uses modeldict from this module.
        In contrast to *GetHumanReadableParms* this function accepts
        single parameter names and does not need the full array of
        parameters.
        *model* - an integer ID of a model
        *name* - the names of the parameters to be translated,
                 order should be same as in
        *parameters* - a list of parameters
        Returns:
        New Units, New Parameters
    """
    stdparms = valuedict[model]
    if len(stdparms) == 5:
        # This means we have extra information on the model
        # Return some human readable stuff
        # Check for list:
        if isinstance(names, basestring):
            names = [names]
            parameters = [parameters]
            retstring = True
        else:
            retstring = False
        # Create new lists
        NewUnits = list()
        NewParameters = list()
        for i in np.arange(len(stdparms[0])):
            for j in np.arange(len(names)):
                if names[j] == stdparms[0][i]:
                    NewUnits.append(stdparms[3][i])
                    NewParameters.append(stdparms[4][i]*parameters[j])
        if retstring == True:
            NewUnits = NewUnits[0]
            NewParameters = NewParameters[0]
        return NewUnits, NewParameters
    else:
        # There is no info about human readable stuff, or it is already human
        # readable.
        return names, parameters


def GetInternalFromHumanReadableParm(model, parameters):
    """ This is the inverse of *GetHumanReadableParms*
        *model* - an integer ID of a model
        *parameters* - a list/array of parameters 
        Returns:
        New Units, New Parameters
    """
    stdparms = valuedict[model]
    if len(stdparms) == 5:
        # This means we have extra information on the model
        # and can convert to internal values
        OldParameters = 1.*np.array(parameters)
        Facors = 1./np.array(stdparms[4])        # inverse
        NewParameters = 1.*OldParameters*Facors
        NewUnits = stdparms[0]
        return NewUnits, NewParameters
    else:
        # There is no info about human readable stuff. The given 
        # parameters have not been converted befor using
        # *GetHumanReadableParms*.
        return stdparms[0], parameters
        

def GetModelType(modelid):
    """ Given a modelid, get the type of model function
        (Confocal, TIR-Conf., TIR-□, User)
    """
    if modelid >= 7000:
        return u"User"
    else:
        shorttype = dict()
        shorttype[u"Confocal (Gaussian)"] = u"Confocal"
        shorttype[u"TIR (Gaussian/Exp.)"] = u"TIR Conf."
        shorttype[u"TIR (□xσ/Exp.)"] = u"TIR □xσ"
        for key in modeltypes.keys():
            mlist = modeltypes[key]
            if mlist.count(modelid) == 1:
                return shorttype[key]
                try:
                    return shorttype[key]
                except:
                    return ""

def GetModelFunctionFromId(modelid):
    return modeldict[modelid][3]
    

def GetModelParametersFromId(modelid):
    return valuedict[modelid][1]


def GetModelFitBoolFromId(modelid):
    return valuedict[modelid][2]


def GetMoreInfo(modelid, Page):
    """ This functino is called by someone who has already calculated
        some stuff or wants to know more about the model he is looking at.
        *modelid* is an ID of a model.
        *Page* is a wx.flatnotebook page.
        Returns:
         More information about a model in form of a list.
    """
    # Background signal average
    bgaverage = None
    # Signal countrate/average:
    # might become countrate - bgaverage
    countrate = Page.traceavg
    # Get the parameters from the current page.
    parms = Page.active_parms[1]
    Info = list()
    if Page.IsCrossCorrelation is False:
        ## First import the supplementary parameters of the model
        ## The order is important for plot normalization and session
        ## saving as of version 0.7.8
        # Try to get the dictionary entry of a model
        # Background information
        if Page.bgselected is not None:
            # Background list consists of items with
            #  [0] average
            #  [1] name
            #  [2] trace
            bgaverage = Page.parent.Background[Page.bgselected][0]
            # Now set the correct countrate
            # We already printed the countrate, so there's no harm done.
            if countrate is not None:
                # might be that there is no countrate.
                countrate = countrate - bgaverage
        try:
            # This function should return all important information
            # that can be calculated from the given parameters.
            func_info = supplement[modelid]
            data = func_info(parms, countrate)
            for item in data:
                Info.append([item[0], item[1]])
        except KeyError:
            # No information available
            pass
        # In case of cross correlation, we don't show this kind of
        # information.
        if Page.traceavg is not None:
            # Measurement time
            duration = Page.trace[-1,0]/1000
            Info.append(["duration [s]", duration])
            # countrate has to be printed before background.
            # Background might overwrite countrate.
            Info.append(["avg. signal [kHz]", Page.traceavg])
    else:
        ## Cross correlation curves usually have two traces. Since we
        ## do not know how to compute the cpp, we will pass the argument
        ## "None" as the countrate.
        ## First import the supplementary parameters of the model
        ## The order is important for plot normalization and session
        ## saving as of version 0.7.8
        # Try to get the dictionary entry of a model
        try:
            # This function should return all important information
            # that can be calculated from the given parameters.
            func_info = supplement[modelid]
            data = func_info(parms, None)
            for item in data:
                Info.append([item[0], item[1]])
        except KeyError:
            # No information available
            pass
        if Page.tracecc is not None:
            # Measurement time
            duration = Page.tracecc[0][-1,0]/1000
            Info.append(["duration [s]", duration])
            # countrate has to be printed before background.
            # Background might overwrite countrate.
            avg0 = Page.tracecc[0][:,1].mean()
            avg1 = Page.tracecc[1][:,1].mean()
            Info.append(["avg. signal A [kHz]", avg0])
            Info.append(["avg. signal B [kHz]", avg1])


    if len(Info) == 0:
        # If nothing matched until now:
        return None
    else:
        return Info


def GetPositionOfParameter(model, name):
    """ Returns an integer corresponding to the position of the label
        of a parameter in the model function
    """
    stdparms = valuedict[model]
    for i in np.arange(len(stdparms[0])):
        if name == stdparms[0][i]:
            return int(i)
    

# Pack all variables
values = list()
# Also create a dictionary, key is modelid
valuedict = dict()
# Pack all models
models = list()
# Also create a dictinary
modeldict = dict()
# A dictionary for supplementary data:
supplement = dict()
# A dictionary for checking for correct variables
verification = dict()


# 6001 6002 6031
AppendNewModel(MODEL_classic_gaussian_2D.Modelarray)

# 6011 6012 6030
AppendNewModel(MODEL_classic_gaussian_3D.Modelarray)

# 6032
AppendNewModel(MODEL_classic_gaussian_3D2D.Modelarray)

# 6013 6014
AppendNewModel(MODEL_TIRF_gaussian_1C.Modelarray)

# 6033
AppendNewModel(MODEL_TIRF_gaussian_3D2D.Modelarray)

# 6034
AppendNewModel(MODEL_TIRF_gaussian_3D3D.Modelarray)

# 6000 6010
AppendNewModel(MODEL_TIRF_1C.Modelarray)

# 6022
AppendNewModel(MODEL_TIRF_2D2D.Modelarray)

# 6020
AppendNewModel(MODEL_TIRF_3D2D.Modelarray)

# 6023
AppendNewModel(MODEL_TIRF_3D3D.Modelarray)

# 6021
AppendNewModel(MODEL_TIRF_3D2Dkin_Ries.Modelarray) 



# Create a list for the differentiation between the models
# This should make everything look a little cleaner
modeltypes = dict()
#modeltypes[u"Confocal (Gaussian)"] = [6001, 6002, 6012, 6011, 6031, 6032, 6030]
#modeltypes[u"TIR (Gaussian/Exp.)"] = [6013, 6033, 6034]
#modeltypes[u"TIR (□xσ/Exp.)"] = [6000, 6010, 6022, 6020, 6023, 6021]

modeltypes[u"Confocal (Gaussian)"] = [6011, 6030, 6002, 6031, 6032]
modeltypes[u"TIR (Gaussian/Exp.)"] = [6014, 6034, 6033]
modeltypes[u"TIR (□xσ/Exp.)"] = [6010, 6023, 6000, 6022, 6020, 6021]
modeltypes[u"User"] = list()




# -*- coding: utf-8 -*-
""" PyCorrFit

    Module tools - channels
    Let the user choose time domains.

    Dimensionless representation:
    unit of time        : 1 ms
    unit of inverse time: 10³ /s
    unit of distance    : 100 nm
    unit of Diff.coeff  : 10 µm²/s
    unit of inverse area: 100 /µm²
    unit of inv. volume : 1000 /µm³

    Copyright (C) 2011-2012  Paul Müller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License 
    along with this program. If not, see <http://www.gnu.org/licenses/>.
"""


import wx
import numpy as np


# Menu entry name
MENUINFO = ["&Data range",
            "Select an interval of lag times to be used for fitting."]

class SelectChannels(wx.Frame):
    def __init__(self, parent):
        # parent is main frame
        self.parent = parent
        # Get the window positioning correctly
        pos = self.parent.GetPosition()
        pos = (pos[0]+100, pos[1]+100)
        wx.Frame.__init__(self, parent=self.parent, title="Data range selection",
                 pos=pos, style=wx.DEFAULT_FRAME_STYLE|wx.FRAME_FLOAT_ON_PARENT)
        ## MYID
        # This ID is given by the parent for an instance of this class
        self.MyID = None
        ## Start drawing
        panel = wx.Panel(self)
        self.panel = panel
        # Page
        self.Page = self.parent.notebook.GetCurrentPage()
        self.Calc_init(self.Page)
        text1 = wx.StaticText(panel, label=u"The lag times τ are stored as an "+
                                           u"array of length ")
        self.textend = wx.StaticText(panel, label="%d." % self.lentau)
        text2 = wx.StaticText(panel,
                              label=u"You may wish to confine this array. "+
                                    u"This can be done here.")
        ##Spincontrols:
        FlexSpinSizer = wx.FlexGridSizer(rows=2, cols=4, vgap=5, hgap=5)
        FlexSpinSizer.Add(wx.StaticText(panel, label="Channels:"))
        self.spinstart = wx.SpinCtrl(panel, -1, initial=self.left, 
                                     min=self.start0, max=self.end0-1)
        FlexSpinSizer.Add(self.spinstart)
        FlexSpinSizer.Add(wx.StaticText(panel, label=" - "))
        self.spinend = wx.SpinCtrl(panel, -1, initial=self.right, 
                                   min=self.start0+1, max=self.end0)
        FlexSpinSizer.Add(self.spinend)
        FlexSpinSizer.Add(wx.StaticText(panel, label="Times [ms]:"))
        self.TextTimesStart = wx.StaticText(panel, label="None")
        FlexSpinSizer.Add(self.TextTimesStart)
        FlexSpinSizer.Add(wx.StaticText(panel, label=" - "))
        self.TextTimesEnd = wx.StaticText(panel, label="None")
        FlexSpinSizer.Add(self.TextTimesEnd)
        # Buttons
        btnapply = wx.Button(panel, wx.ID_ANY, 'Apply')
        btnapplyall = wx.Button(panel, wx.ID_ANY, 'Apply to all pages')
        self.ButtonApply = btnapply
        self.ButtonApplyAll = btnapplyall
        self.Bind(wx.EVT_BUTTON, self.OnApply, btnapply)
        self.Bind(wx.EVT_BUTTON, self.OnApplyAll, btnapplyall)
        self.Bind(wx.EVT_SPINCTRL, self.OnChangeChannels, self.spinend)
        self.Bind(wx.EVT_SPINCTRL, self.OnChangeChannels, self.spinstart)
        # Checkbox
        self.fixcheck = wx.CheckBox(panel, -1,
             label="Fix current channel selection for all pages.")
        self.Bind(wx.EVT_CHECKBOX, self.OnCheckbox, self.fixcheck)
        # Text
        channelsel = "Leave this window open for a fixed selection."
        text3 = wx.StaticText(panel, label=channelsel)
        # Sizer
        topSizer = wx.BoxSizer(wx.VERTICAL)
        buttonsizer = wx.BoxSizer(wx.HORIZONTAL)
        buttonsizer.Add(btnapply, 1)
        buttonsizer.Add(btnapplyall, 1)
        text1sizer = wx.BoxSizer(wx.HORIZONTAL)
        text1sizer.Add(text1)
        text1sizer.Add(self.textend)
        topSizer.Add(text1sizer)
        topSizer.Add(text2)
        topSizer.AddSpacer(5)
        topSizer.Add(FlexSpinSizer)
        topSizer.Add(self.fixcheck)
        topSizer.Add(text3)
        topSizer.AddSpacer(5)
        topSizer.Add(buttonsizer)
        panel.SetSizer(topSizer)
        topSizer.Fit(self)
        self.SetMinSize(topSizer.GetMinSizeTuple())
        # Get times.
        self.OnChangeChannels()
        #Icon
        if parent.MainIcon is not None:
            wx.Frame.SetIcon(self, parent.MainIcon)
        # Show window
        self.Show(True)
        self.OnPageChanged(self.Page)


    def Calc_init(self, parent):
        ## Variables
        # Parent should be the fitting panel -
        # The tab, where the fitting is done.
        self.Page = parent
        if self.Page == None:
            # dummy info
            taufull = np.arange(100)
            self.left = self.right = None
            self.panel.Disable()
        else:
            self.left = self.Page.startcrop     # starting position
            self.right = self.Page.endcrop      # ending position
            if self.Page.dataexpfull is not None:
                taufull = self.Page.dataexpfull[:,0]
            else:
                # then we only have tau
                taufull = self.Page.taufull
        self.lentau = len(taufull)
        self.start0 = 0                     # left border of interval
        # The interval starts at 0!
        self.end0 = self.lentau - 1         # right border of interval 
        if self.left is None or self.left > self.end0:
            # This means, that either left = right = None
            # or the dataexp-array is too small
            self.left = self.start0
        if self.right is None:
            # set the maximum possible value
            self.right = self.end0
        else:
            self.right -=1


    def OnApply(self, event=None):
        self.SetValues()
        self.Page.PlotAll()


    def OnApplyAll(self, event=None):
        start = self.spinstart.GetValue()
        end = self.spinend.GetValue() + 1 # +1, [sic]
        if start > end:
            # swap the variables, we are not angry at the user
            start, end = end, start
        # Get all the Pages
        N = self.parent.notebook.GetPageCount()
        for i in np.arange(N):
            # Set Page 
            Page = self.parent.notebook.GetPage(i)
            # Find out maximal length
            if Page.dataexpfull is not None:
                maxlen = len(Page.dataexpfull[:,0])
            else:
                # then we only have tau
                maxlen = len(Page.taufull)
            # Use the smaller one of both, so we do not get an
            # index out of bounds error
            Page.endcrop = min(end, maxlen)
            Page.startcrop = start*(start < maxlen - 1 )
            Page.PlotAll()
        # Page.PlorAll() calls this function. This results in the wrong data
        # being displayed in an open "View Info" Window. We call it again.
        self.parent.OnFNBPageChanged()


    def OnChangeTimes(self, e=None):
        """ Called, whenever data range in seconds is changed. This updates
            the data range in channels in the window.
            This function might be used in later versions of PyCorrFit.
        """
        pass


    def OnChangeChannels(self, e=None):
        """ Called, whenever data range in channels is changed. This updates
            the data range in seconds in the window.
        """
        if self.Page == None:
            return
        N = len(self.Page.taufull)
        start = self.spinstart.Value
        end = self.spinend.Value
        # If the initial boundaries are outside of the experimental
        # data array of length N, change the start and end variables.
        start = start*(start < N-2)
        end = min(end, N-1)
        t1 = 1.*self.Page.taufull[start]
        t2 = 1.*self.Page.taufull[end]
        self.TextTimesStart.SetLabel("%.4e" % t1)
        self.TextTimesEnd.SetLabel("%.4e" % t2)
        self.OnCheckbox()


    def OnCheckbox(self, event=None):
        """ Set the correct value in the spincontrol, if the checkbox
            is not checked.
        """
        state = self.fixcheck.GetValue()
        if state == True:
            self.OnApplyAll()
            self.ButtonApply.Disable()
            self.ButtonApplyAll.Disable()
        else:
            self.ButtonApply.Enable()
            self.ButtonApplyAll.Enable()
        #self.OnPageChanged(self.Page)
            

    def OnClose(self, event=None):
        self.parent.toolmenu.Check(self.MyID, False)
        self.parent.ToolsOpen.__delitem__(self.MyID)
        self.Destroy()


    def OnPageChanged(self, page, trigger=None):
        # We do not need the *Range* Commands here yet.
        # We open and close the SelectChannelsFrame every time we
        # import some data.
        #
        if trigger in ["parm_batch", "fit_batch", "page_add_batch"]:
            return
        # Check if we have a fixed channel selection
        if self.parent.notebook.GetPageCount() == 0:
            self.panel.Disable()
        else:
            self.panel.Enable()
            # There is a page. We may continue.
            state = self.fixcheck.GetValue()
            if state == True:
                # We do not need to run Calc_init
                self.Page = page
                self.SetValues()
                self.Page.PlotAll(event="init")
            else:
                # We will run it
                self.Calc_init(page)
                self.spinstart.SetRange(self.start0, self.end0-1)
                self.spinstart.SetValue(self.left)
                self.spinend.SetRange(self.start0+1, self.end0)
                self.spinend.SetValue(self.right)
                self.textend.SetLabel("%d." % self.lentau)
                self.OnChangeChannels()



    def SetValues(self):
        start = self.spinstart.GetValue()
        end = self.spinend.GetValue()
        if start > end:
            # swap the variables, we are not angry at the user
            start, end = end, start
        self.Page.startcrop = start
        self.Page.endcrop = end + 1 # +1, because arrays are accessed like this
